/*
 *  LinePol - ARTIST
 *
 *  Created by Rolf Kuiper
 *
 *	All rights reserved.
 *
 */


#define DEBUG_SINGLE_ITERATION NO


#if LINEPOLARIZATION == ON

//
// Vector Algebra:
//
double vectoralgebra_DotProduct(double *, double *);
double vectoralgebra_Abs(double *);
void vectoralgebra_Normalize(double *);
void vectoralgebra_Project(double *, double *, double *);
void vectoralgebra_CrossProduct(double *, double *, double *);

int vectoralgebra_CheckUnitVector(double *);
int vectoralgebra_CheckPerpendicularity(double *, double *);


#define BOHRMAG (9.274009e-24) // in J T^-1

int    *linepolarization_SplittedLevels;
int    linepolarization_NSplittedLevels;

int    DebugLinePolarization;
FILE   *DebugLinePolarizationFile;

extern int **dirs;
extern double ***inidirs;

void   magneticfield(double,double,double,double *);

int    linepolarization_AreMagneticSublevelsTreatedExplicitly(int);
int    linepolarization_GetEnergyLevelNumber(int, int, molData *, int);

void   linepolarization_sourceFunc_line(double *, double *, double *, double *, molData *, double, struct grid *, int, int, int, double);
void   linepolarization_sourceFunc_cont(double *, double *, double *, double *, struct grid *, int, int, int, molData *);

double linepolarization_GetClebschGordonCoefficient(int, int, int);
void   linepolarization_GetPropagationDirection_photon(double *, struct grid *, int, int);
void   linepolarization_GetPropagationDirection_raytrace(double *, image *, int);
double linepolarization_GetAngleBetweenPropagationDirectionAndMagneticField(struct grid *, int, double *);
void   linepolarization_PolarizationFrame2CartesianFrame(struct grid*, int, double*, double, double, double*);
void   linepolarization_PolarizationFrame2CartesianFrame_snu_and_tau_simultaneously(struct grid*, int, double*, double, double, double, double, double*, double*);
void   linepolarization_CartesianFrame2PolarizationFrame(struct grid*, int, double*, double*, double*, double*);
double Lande(int, int, int);
double DopplerLinewidth(double, double, double, molData *, int, int);
double ZeemanLinewidth(double);
double NaturalLinewidth(molData *, int, int);
#endif












#ifdef INTERNALSOURCES
// Constants:
#define SIGMA   (2.0*pow(PI,5)*pow(KBOLTZ,4) /15.0/pow(HPLANCK,3)/pow(CLIGHT,2)) // in W m^-2 K^-4
#define RSOL    (0.004652*AU) // in m
#define TSOL    5800.0 // in K
#define LSOL    (4.0*PI*SIGMA*pow(RSOL,2)*pow(TSOL,4)) // in J s^-1
#define MSOL    (1.9891e+30) // in kg
#define YR      (365.25*24.0*60.0*60.0) // in s

int DebugInternalSources;
FILE *DebugInternalSourcesFile;

int dointernalsources;
int NSources;
double* TSource;
double* RSource;
double* LSource;
double* PositionRadiusSource, *PositionThetaSource, *PositionPhiSource;
double* PositionXSource, *PositionYSource, *PositionZSource;
void internalsources(inputPars *);
#endif
