/*
 *  LinePol - ARTIST
 *
 *  Created by Rolf Kuiper
 *
 *	All rights reserved.
 *
 */

#include "lime.h"
#if LINEPOLARIZATION == ON
#include "linepolarization.h"

void linepolarization_sourceFunc_line(double *jnu_para, double *jnu_perp, double *alpha_para, double *alpha_perp, molData *m, double vfac,struct grid *g,int pos,int ispec, int iline, double angle_nB){
	
	int Jup, Jdown, Mup, Mdown, dM;
	double jnu, alpha;

	jnu = 0;
	alpha = 0;
    
	sourceFunc_line(&jnu,&alpha,m,vfac,g,pos,ispec,iline);

	// for para and perp splitting, cp. Cortes et al., eq. (8):
	jnu *= 0.5;
		
	Jup   = m[ispec].lau[iline];
	Jdown = m[ispec].lal[iline];
	Mup   = m[ispec].mau[iline];
	Mdown = m[ispec].mal[iline];
	dM = abs(Mup-Mdown);
	
	if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jup) || linepolarization_AreMagneticSublevelsTreatedExplicitly(Jdown)){
		jnu   *= 2.0*Jup+1.0;
		alpha *= 2.0*Jup+1.0;
		
		if(dM == 0){
			*jnu_para   += pow(sin(angle_nB), 2) * jnu;
			*alpha_para += pow(sin(angle_nB), 2) * alpha;
			*jnu_perp   += 0.0;
			*alpha_perp += 0.0;
		}
		else if(dM == 1){
			*jnu_para   += 0.5 * pow(cos(angle_nB), 2) * jnu;
			*alpha_para += 0.5 * pow(cos(angle_nB), 2) * alpha;
			*jnu_perp   += 0.5 * jnu;
			*alpha_perp += 0.5 * alpha;
		}
		else{
			bail_out("ERROR: dM not equal to 1 or 0.");
		}
	}
	else{
		*jnu_para   += jnu;
		*jnu_perp   += jnu;
		*alpha_para += alpha;
		*alpha_perp += alpha;
	}
	
	return;
}


void linepolarization_sourceFunc_cont(double *jnu_para, double *jnu_perp, double *alpha_para, double *alpha_perp, struct grid *g, int pos, int ispec, int iline, molData *m){
	
	double jnu, alpha;
	
	jnu = 0;
	alpha = 0;

	sourceFunc_cont(&jnu,&alpha,g,pos,ispec,iline);
		
	*jnu_para   += 0.5*jnu;
	*jnu_perp   += 0.5*jnu;
	*alpha_para += alpha;
	*alpha_perp += alpha;
	
	return;
}

#endif
