/*
 *  LinePol - ARTIST
 *
 *  Created by Rolf Kuiper
 *
 *	All rights reserved.
 *
 */

#include "lime.h"
#if LINEPOLARIZATION == ON
#include "linepolarization.h"


double vectoralgebra_DotProduct(double *a, double *b){
	return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
}

void vectoralgebra_CrossProduct(double *a, double *b, double *a_cross_b){	
	a_cross_b[0] = a[1]*b[2] - a[2]*b[1];
	a_cross_b[1] = a[2]*b[0] - a[0]*b[2];
	a_cross_b[2] = a[0]*b[1] - a[1]*b[0];
}

// length of vector
double vectoralgebra_Abs(double *a){
	return sqrt(vectoralgebra_DotProduct(a, a));
}

// transform vector into a unit vector, maintaining the direction
void vectoralgebra_Normalize(double *a){
	double abs_a;
	abs_a = vectoralgebra_Abs(a);
	a[0] /= abs_a;
	a[1] /= abs_a;
	a[2] /= abs_a;	
}

// orthogonal projection of vector b onto the direction of vector a
void vectoralgebra_Project(double *b, double *a, double *ba){
	double abs_ba;
	vectoralgebra_Normalize(a);
	abs_ba = vectoralgebra_DotProduct(a, b);
	ba[0] = abs_ba * a[0];
	ba[1] = abs_ba * a[1];
	ba[2] = abs_ba * a[2];
}


// checks:
int vectoralgebra_CheckPerpendicularity(double *a, double *b){
	double precision = 1e-14;
	double a_dot_b;
	a_dot_b = vectoralgebra_DotProduct(a, b);
	if(fabs(a_dot_b) < precision) return 1;
	else return 0;
}

int vectoralgebra_CheckUnitVector(double *a){
	double precision = 1e-14;
	double abs_a;
	abs_a = vectoralgebra_Abs(a);
	if(fabs(1.0 - abs_a) < precision) return 1;
	else return 0;
}


#endif
