/*
 *  grid.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  Created by Christian Brinch on 11/16/06.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@strw.leidenuniv.nl>
 *  Sterrewacht Leiden
 *  Leiden University
 *	All rights reserved.
 *
 */

#include <qhull/qhull_a.h>
#include "lime.h"

void 
gridAlloc(inputPars *par, struct grid **g){
	int i;
	double temp[99];	
  	
	*g=malloc(sizeof(struct grid)*(par->pIntensity+par->sinkPoints));	
	memset(*g, 0., sizeof(struct grid) * (par->pIntensity+par->sinkPoints));

	if(par->pregrid) par->collPart=1;
	else{
	  for(i=0;i<99;i++) temp[i]=-1;
	  density(AU,AU,AU,temp);
	  i=0;
	  par->collPart=0;
	  while(temp[i++]>-1) par->collPart++;
	}
	
	for(i=0;i<(par->pIntensity+par->sinkPoints); i++){
	  (*g)[i].dens=malloc(sizeof(double)*par->collPart);
	  (*g)[i].nmol=malloc(sizeof(double)*par->nSpecies);
	  (*g)[i].t[0]=-1;
	  (*g)[i].t[1]=-1;
	}
}


void
voronoi(inputPars *par, struct grid *g, const gsl_rng *ran){
  int j,k,id;
//  int t,i,j,k,id,best,b;
  char flags[255];			    
  boolT ismalloc = False;		
  facetT *neighbor, **neighborp;  	
//  facetT *facet, **facetp, *neighbor, **neighborp;  
  vertexT *vertex;	
//	ridgeT *ridge, **ridgep;
  coordT *pt_array;  
//  double tmp, *angle, *point,len,area,pt_theta,pt_phi,x,y,z;
  
	printf("%d %e\n", g[0].numNeigh, g[0].x[0]);
	
//	for(k=0;k<par->pIntensity;k++){
	for(k=0;k<1;k++){
//		printf("%d\n", g[k].numNeigh);
		pt_array=malloc(2*(g[k].numNeigh+1)*sizeof(coordT));
		pt_array[0]=g[k].x[0];
		pt_array[1]=g[k].x[1];
		pt_array[2]=g[k].x[2];    
    for(j=1;j<(g[k].numNeigh+1);j++) {
			pt_array[3*j+0]=g[g[k].neigh[j-1]].x[0];
			pt_array[3*j+1]=g[g[k].neigh[j-1]].x[1];
			pt_array[3*j+2]=g[g[k].neigh[j-1]].x[2];
		}
		
		sprintf(flags,"qhull v Qbb T0");
		if (!qh_new_qhull(3, g[k].numNeigh+1, pt_array, ismalloc, flags, NULL, NULL)) {
			qh_setvoronoi_all();
			qh_findgood_all (qh facet_list);
			qh_getarea(qh facet_list);

			FORALLvertices {
				id=qh_pointid(vertex->point);
				if(id==0) {
					printf("%d %e\n", id, vertex->point[0]);
					FOREACHneighbor_ (vertex){
						printf("%e\n", neighbor->f.area);
					} 
				}
			}
					
		
		
		}
		free(pt_array);
	}
	exit(0);
	
		
		
		
		
		
		
		
		
		
		
		
	/*
  pt_array=malloc(DIM*par->ncell*sizeof(coordT));
  for(i=0;i<par->ncell;i++) {
    for(j=0;j<DIM;j++) {
	  pt_array[i*DIM+j]=g[i].x[j];
	}
  }
		
  sprintf(flags,"qhull v s Qbb T0");
  
  if (!qh_new_qhull(DIM, par->ncell, pt_array, ismalloc, flags, NULL, NULL)) {
  	qh_setvoronoi_all();
	  FORALLvertices {
		  id=qh_pointid(vertex->point);
    	t=0;
		  FOREACHneighbor_ (vertex) {
		    if(!neighbor->upperdelaunay) t++;
		  }
		  point=malloc(3*t*sizeof(double));
		  angle=malloc(t*sizeof(double));
	    j=0;
		  FOREACHneighbor_ (vertex){
		    if(!neighbor->upperdelaunay) {
			    point[j++]=neighbor->center[0];
			    point[j++]=neighbor->center[1];	
			    point[j++]=neighbor->center[2];	
		    }  
		  }

		for(j=0;j<t;j++){
		  len=sqrt(pow(point[3*j]-g[id].x[0],2)+pow(point[3*j+1]-g[id].x[1],2)+pow(point[3*j+2]-g[id].x[2],2));
		  angle[j]=(point[3*j]-vertex->point[0])/len;
		  if((point[3*j+1]-vertex->point[1])>=0.0) angle[j]=2.-angle[j];
		}
		for(i=t-1; i>=0; i--) {
		  for(j=1; j<=i; j++) {
		    if(angle[j-1]>angle[j]) {
				tmp=point[2*(j-1)];
				point[2*(j-1)]=point[2*j];
				point[2*j]=tmp;
				
				tmp=point[2*(j-1)];
				point[2*(j-1)]=point[2*(j)+1];
				point[2*(j)+1]=tmp;

				tmp=angle[j-1];
				angle[j-1]=angle[j];
				angle[j]=tmp;
			}
		  }
		}


 //		for(j=0;j<t;j++) printf("%d %d %e %e\n", j,t,point[2*j],point[2*j+1]);
		free(point);
		free(angle);
		}
  }
  free(pt_array);
*/
}

void
qhull(inputPars *par, struct grid *g){
  int i,j,k,id;
  char flags[255];			    
  boolT ismalloc = False;		
  facetT *facet;				
  vertexT *vertex,**vertexp;	
  coordT *pt_array;  
  int simplex[DIM];				

  pt_array=malloc(DIM*sizeof(coordT)*par->ncell);

  for(i=0;i<par->ncell;i++) {
    for(j=0;j<DIM;j++) {
	  pt_array[i*DIM+j]=g[i].x[j];
	}
  }
  sprintf(flags,"qhull d Qbb T0");
  if (!qh_new_qhull(DIM, par->ncell, pt_array, ismalloc, flags, NULL, NULL)) {
    /* Identify points */
    FORALLvertices {
      id=qh_pointid(vertex->point);
			g[id].numNeigh=qh_setsize(vertex->neighbors);
			free(g[id].neigh);
			g[id].neigh=malloc(sizeof(int)*g[id].numNeigh);
			memset(g[id].neigh, 0, sizeof(int) * g[id].numNeigh);   
			for(k=0;k<g[id].numNeigh;k++) {
        g[id].neigh[k]=-1;
      }
    }		
		/* Identify neighbors */
		FORALLfacets {
			if (!facet->upperdelaunay) {
				j=0;
				FOREACHvertex_ (facet->vertices) simplex[j++]=qh_pointid(vertex->point);
				for(i=0;i<DIM+1;i++){
					for(j=0;j<DIM+1;j++){
						k=0;
						if(i!=j){
							while(g[simplex[i]].neigh[k]>-1 && g[simplex[i]].neigh[k] != simplex[j]) {
								k++;
							}	  
							g[simplex[i]].neigh[k]=simplex[j];	
						}
					}
				}
			}				
    }
		
		
  }
 
  for(i=0;i<par->ncell;i++){ 
    j=0;
    for(k=0;k<g[i].numNeigh;k++){
      if(g[i].neigh[k]>-1) j++;
    }
    g[i].numNeigh=j;
  }
  qh_freeqhull(!qh_ALL);
}
	
void
distCalc(inputPars *par, struct grid *g){
  int i,k,l;

  for(i=0;i<par->ncell;i++){		
    free(g[i].dir);
	free(g[i].ds);
	g[i].dir=malloc(sizeof(point)*g[i].numNeigh);
	g[i].ds =malloc(sizeof(double)*g[i].numNeigh);
	memset(g[i].dir, 0., sizeof(point) * g[i].numNeigh);  
	memset(g[i].ds, 0., sizeof(double) * g[i].numNeigh);  
	for(k=0;k<g[i].numNeigh;k++){		
	  for(l=0;l<3;l++) g[i].dir[k].x[l] = g[g[i].neigh[k]].x[l] - g[i].x[l];
	  g[i].ds[k]=sqrt(pow(g[i].dir[k].x[0],2)+pow(g[i].dir[k].x[1],2)+pow(g[i].dir[k].x[2],2));
	  for(l=0;l<3;l++) g[i].dir[k].xn[l] = g[i].dir[k].x[l]/g[i].ds[k];		
	}
	g[i].nphot=ininphot*g[i].numNeigh;
  }
}

void 
write_VTK_unstructured_Points(inputPars *par, struct grid *g){
  FILE *out;
  double vel[3];
  int i,j,l=0;
  char flags[255];			    
  boolT ismalloc = False;		
  facetT *facet;				
  vertexT *vertex,**vertexp;	
  coordT *pt_array;  

  pt_array=malloc(sizeof(coordT)*DIM*par->ncell);  

  for(i=0;i<par->ncell;i++) {
    for(j=0;j<DIM;j++) {
	  pt_array[i*DIM+j]=g[i].x[j];
	}
  }
  
  out = fopen(par->gridfile,"w");
  fprintf(out,"# vtk DataFile Version 3.0\n");
  fprintf(out,"Lime grid\n");
  fprintf(out,"ASCII\n");
  fprintf(out,"DATASET UNSTRUCTURED_GRID\n");
  fprintf(out,"POINTS %d float\n",par->ncell);
  for(i=0; i<par->ncell; i++) {
    fprintf(out,"%e %e %e\n", g[i].x[0], g[i].x[1], g[i].x[2]);
  }
  fprintf(out, "\n");
 
  sprintf(flags,"qhull d Qbb T0");
  
  if (!qh_new_qhull(DIM, par->ncell, pt_array, ismalloc, flags, NULL, NULL)) {
	FORALLfacets {
	  if (!facet->upperdelaunay) l++;
	}
    fprintf(out,"CELLS %d %d\n",l, 5*l);
	FORALLfacets {
	  j=0;
	  if (!facet->upperdelaunay) {
	    fprintf(out,"4 ");
		FOREACHvertex_ (facet->vertices) {
		  fprintf(out, "%d ", qh_pointid(vertex->point));
		}
		fprintf(out, "\n");
	  }  				
    }
  }
  qh_freeqhull(!qh_ALL);
  fprintf(out,"\nCELL_TYPES %d\n",l);
  for(i=0;i<l;i++){
	fprintf(out, "9\n");
  }
  fprintf(out,"POINT_DATA %d\n",par->ncell);
  fprintf(out,"SCALARS Scalars float 3\n");
  fprintf(out,"LOOKUP_TABLE default\n");
  for(i=0;i<par->ncell;i++){
	fprintf(out, "%e\n", g[i].dens[0]);
  }
  for(i=0;i<par->ncell;i++){
	fprintf(out, "%e\n", g[i].t[0]);
  }
  for(i=0;i<par->ncell;i++){
	fprintf(out, "%e\n", g[i].nmol[0]);
  }
  fprintf(out,"VECTORS vectors float\n");
  for(i=0;i<par->ncell;i++){
	velocity(g[i].x[0],g[i].x[1],g[i].x[2],vel);
	fprintf(out, "%e %e %e\n", vel[0],vel[1],vel[2]);
  }
	

  fclose(out);
  free(pt_array);
}

void
dumpGrid(inputPars *par, struct grid *g){
//  voronoi(par, g); 
  if(par->gridfile) write_VTK_unstructured_Points(par, g);
}	

void
getArea(inputPars *par, struct grid *g, const gsl_rng *ran){
	int i,j,k,b=-1;
    double *angle,best;
	/*	double wsum; */
	double x,y,z,pt_phi,pt_theta;
	
	/* Lots of circles approach -- badly broken, needs to be fixed  */
	/*
	 for(i=0;i<par->pIntensity;i++){
	 angle=malloc(sizeof(double)*g[i].numNeigh);
	 g[i].w=malloc(sizeof(double)*g[i].numNeigh);
	 for(k=0;k<g[i].numNeigh;k++){
	 best=0;
	 for(j=0;j<g[i].numNeigh;j++){
	 angle[j]=( g[i].dir[k].xn[0]*g[i].dir[j].xn[0]
	 +g[i].dir[k].xn[1]*g[i].dir[j].xn[1]
	 +g[i].dir[k].xn[2]*g[i].dir[j].xn[2]);
	 if(angle[j]>best && angle[j]<1){
	 best=angle[j];
	 b=j;
	 }
	 }
	 g[i].w[k]=PI*pow(tan(acos(best)),2);
	 wsum+=g[i].w[k];
	 }
	 for(k=0;k<g[i].numNeigh;k++) g[i].w[k]=g[i].w[k]/wsum;
	 free(angle);
	 }
	 */
	
	
	for(i=0;i<par->pIntensity;i++){
		angle=malloc(sizeof(double)*g[i].numNeigh);
		g[i].w=malloc(sizeof(double)*g[i].numNeigh);
		memset(g[i].w, 0, sizeof(double) * g[i].numNeigh);   
		for(k=0;k<10000;k++){
			// TODO: at Christian: This routine does NOT give random dirs, but introduces a focus onto polar regions
			pt_theta=gsl_rng_uniform(ran)*2*PI;
			pt_phi=gsl_rng_uniform(ran)*PI;
			x=cos(pt_theta)*sin(pt_phi);
			y=sin(pt_theta)*sin(pt_phi);
			z=cos(pt_phi);
			best=-1;
			for(j=0;j<g[i].numNeigh;j++){
				angle[j]=( x*g[i].dir[j].xn[0]
						  +y*g[i].dir[j].xn[1]
						  +z*g[i].dir[j].xn[2]);
				if(angle[j]>best){
					best=angle[j];
					b=j;
				}  
			}
			if(b<0) printf("not supposed to happen\n");
			g[i].w[b]+=0.0001;
			b=-1;
		}
		free(angle);
	}
}

void
getMass(inputPars *par, struct grid *g, const gsl_rng *ran){
	double mass=0., d, boxvol,home,mind,in;
	double x,y,z,vol, tmp;
	int i,k,j;
	
	for(i=0;i<par->pIntensity;i++){
	    d=0.;
		for(k=0;k<g[i].numNeigh;k++){
			if(g[i].ds[k]>d) d=g[i].ds[k];
		}
		
		boxvol=pow(2*d,3);
		
		in=0.;
		for(j=0;j<10000;j++){
			x=((gsl_rng_uniform(ran)*2)-1.)*d+g[i].x[0];
		    y=((gsl_rng_uniform(ran)*2)-1.)*d+g[i].x[1];
			z=((gsl_rng_uniform(ran)*2)-1.)*d+g[i].x[2];
			
			home=sqrt(pow(x-g[i].x[0],2)+pow(y-g[i].x[1],2)+pow(z-g[i].x[2],2));
			mind=1e30;
			for(k=0;k<g[i].numNeigh;k++){
				d=sqrt(pow(x-g[g[i].neigh[k]].x[0],2)+pow(y-g[g[i].neigh[k]].x[1],2)+pow(z-g[g[i].neigh[k]].x[2],2));
				if(d<mind) mind=d;
			}
			if(home<mind) in+=1;
		}
		if(in > 0){
		  vol=boxvol*in/10000.;
          //printf("%d: vol = %e\n", i, vol);
		} else {
		  printf("superfail\n");
		  if(!silent) bail_out("Voronoi cell volume cannot be determined");
		}
		tmp=0;
		for(j=0;j<par->collPart;j++) tmp+=g[i].dens[j];
		mass+=vol*tmp;
	    //printf("%d %e %e %e %e\n", i, tmp, vol*g[i].nmol[0]*2.37*1.67e-27/1.989e30, g[i].t[0], (g[i].dens[0]+g[i].dens[1]));
	}
	if(!silent) quotemass(mass*2.37*1.67e-27/1.989e30);
}


void 
buildGrid(inputPars *par, struct grid *g){

#ifdef INTERNALSOURCES
	internalsources(par);
#endif

	double scale;					/* Parameter to distribute surface point 	*/
	double lograd;				/* The logarithm of the model radius			*/
	double logmin;				/* Logarithm of par->minScale							*/
	double r,theta,phi,x,y,z;	/* Coordinates												*/
	double temp,*abun;
	int k=0,j,i;					/* counters																*/
	int flag;
	
	const gsl_rng *ran = gsl_rng_alloc(gsl_rng_ranlxs2);	/* Random number generator */
	gsl_rng_set(ran,time(0));

	abun=malloc(sizeof(double)*par->nSpecies);

	lograd=log10(par->radius);
	logmin=log10(par->minScale);

	/* Sample pIntensity number of points */
	for(k=0;k<par->pIntensity;k++){
		temp=gsl_rng_uniform(ran);
		flag=0;
		/* Pick a point and check if we like it or not */
		do{
		  if(par->sampling==0){
		    r=pow(10,logmin+gsl_rng_uniform(ran)*(lograd-logmin));
		    theta=2.*PI*gsl_rng_uniform(ran);
		    phi=PI*gsl_rng_uniform(ran);
#ifdef INTERNALSOURCES
		    if(k < NSources){
		    	r=PositionRadiusSource[k];
		    	theta=PositionThetaSource[k];
		    	phi=PositionPhiSource[k];
		    }
#endif
		    x=r*cos(theta)*sin(phi);
		    y=r*sin(theta)*sin(phi);
	        if(DIM==3) z=r*cos(phi);
		    else z=0.;
		  } else if(par->sampling==1){
		    x=(2*gsl_rng_uniform(ran)-1)*par->radius;
		    y=(2*gsl_rng_uniform(ran)-1)*par->radius;
		    if(DIM==3) z=(2*gsl_rng_uniform(ran)-1)*par->radius;
		  	else z=0;
#ifdef INTERNALSOURCES
		    if(k < NSources){
		    	x=PositionXSource[k];
		    	y=PositionYSource[k];
		    	z=PositionZSource[k];
		    }
#endif
		  } else {
		  	if(!silent) bail_out("Don't know how to sample model");
			exit(1);
		  }
		  if(sqrt(x*x+y*y+z*z)<par->radius) flag=pointEvaluation(par,temp,x,y,z);
		} while(!flag); 
		/* Now pointEvaluation has decided that we like the point */
		
		/* Assign values to the k'th grid point */
		/* I don't think we actually need to do this here... */
		g[k].id=k;
	    g[k].x[0]=x;	  
		g[k].x[1]=y;
	    if(DIM==3) g[k].x[2]=z;
	  	else g[k].x[2]=0.;

		g[k].sink=0;
		/* This next step needs to be done, even though it looks stupid */
		g[k].dir=malloc(sizeof(point)*1);
		g[k].ds =malloc(sizeof(point)*1);
		g[k].neigh =malloc(sizeof(int)*1);
		if(!silent) progressbar((double) k/((double)par->pIntensity-1), 4);	
	}
    /* end model grid point assignment */
    

	/* Add surface sink particles */
	for(i=0;i<par->sinkPoints;i++){
		x=2*gsl_rng_uniform(ran)-1.;
		y=2*gsl_rng_uniform(ran)-1.;
		if(DIM==3) z=2*gsl_rng_uniform(ran)-1.;
		else z=0.;
		if(x*x+y*y+z*z<1){
			scale=par->radius*sqrt(1/(x*x+y*y+z*z));
			g[k].id=k;
			g[k].x[0]=scale*x;
			g[k].x[1]=scale*y;
			g[k].x[2]=scale*z;			
			g[k].sink=1;
			g[k].dens[0]=1e-30;
			g[k].t[0]=par->tcmb;
			g[k].t[1]=par->tcmb;
			g[k++].dopb=0.;
		} else i--;		
	}	
	/* end grid allocation */
	
	qhull(par, g);	
	distCalc(par, g);	
//	voronoi(par, g,ran); 
	
	smooth(par,g);
	
	for(i=0;i<par->pIntensity;i++){
		density(g[i].x[0],g[i].x[1],g[i].x[2],g[i].dens);
		temperature(g[i].x[0],g[i].x[1],g[i].x[2],g[i].t);
		doppler(g[i].x[0],g[i].x[1],g[i].x[2],&g[i].dopb);	
		abundance(g[i].x[0],g[i].x[1],g[i].x[2],abun);
#ifdef INTERNALSOURCES
		if(i < NSources){
			g[i].t[0] = TSource[i]; // kinetic gas temperature
			g[i].t[1] = TSource[i]; // dust temperature
			g[i].dens[0] = 1e+10*1e+16; // TODO: scale this to the maximum density in the domain (currently 1e+16) or a meaningful absolute value
			for(j=0;j<par->nSpecies;j++) abun[j] = 0;
			if(DebugInternalSources) fprintf(DebugInternalSourcesFile, "Source %d: g[i].dens[0] = %e\n", i, g[i].dens[0]);
		}
		if(DebugInternalSources) fclose(DebugInternalSourcesFile);
#endif
		for(j=0;j<par->nSpecies;j++) g[i].nmol[j]=abun[j]*g[i].dens[0];
	}

	dumpGrid(par,g);
//#if inidirflag == 1
	getArea(par,g, ran);
//#endif
	getMass(par,g, ran);
	getVelosplines(par,g);
	if(!silent) done(5);

}


