/*
 *  io.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  This extension file to LIME was created by Rolf Kuiper on 2011/05.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@strw.leidenuniv.nl>
 *  Sterrewacht Leiden
 *  Leiden University
 *	All rights reserved.
 *
 */

#include "lime.h"



void
write_grid_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	int i,k;
	sprintf(out_filename, "%s/Output/grid_ascii.out", modelfolder);
	out = fopen(out_filename, "w");
	for(i = 0; i < par->ncell; i++){
		fprintf(out, "%e %e %e %d %d ", g[i].x[0], g[i].x[1], g[i].x[2], g[i].sink, g[i].numNeigh);
		for(k = 0; k < g[i].numNeigh; k++){
			fprintf(out, "%d ", g[i].neigh[k]);
		}
		fprintf(out, "\n");
	}
	fclose(out);
}

void
write_density_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	int i;
	sprintf(out_filename, "%s/Output/density_ascii.out", modelfolder);
	out = fopen(out_filename, "w");
	for(i = 0; i < par->ncell; i++){
		fprintf(out, "%e\n", g[i].dens[0]);
	}
	fclose(out);
}

void
write_temperature_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	int i;
	sprintf(out_filename, "%s/Output/temperature0_ascii.out", modelfolder);
	out = fopen(out_filename, "w");
	for(i = 0; i < par->ncell; i++){
		fprintf(out, "%e\n", g[i].t[0]);
	}
	fclose(out);
	sprintf(out_filename, "%s/Output/temperature1_ascii.out", modelfolder);
	out = fopen(out_filename, "w");
	for(i = 0; i < par->ncell; i++){
		fprintf(out, "%e\n", g[i].t[1]);
	}
	fclose(out);
}

void
write_nmol_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	int i;
	sprintf(out_filename, "%s/Output/nmol_ascii.out", modelfolder);
	out = fopen(out_filename, "w");
	for(i = 0; i < par->ncell; i++){
		fprintf(out, "%e\n", g[i].nmol[0]);
	}
	fclose(out);
}

void
write_velocity_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	int i;
	double vel[3];
	sprintf(out_filename, "%s/Output/velocity_ascii.out", modelfolder);
	out = fopen(out_filename, "w");
	for(i = 0; i < par->ncell; i++){
		velocity(g[i].x[0],g[i].x[1],g[i].x[2], vel);
		fprintf(out, "%e %e %e\n", vel[0], vel[1], vel[2]);
	}
	fclose(out);
}

void
write_doppler_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	int i;
	sprintf(out_filename, "%s/Output/doppler_ascii.out", modelfolder);
	out = fopen(out_filename, "w");
	for(i = 0; i < par->ncell; i++){
		fprintf(out, "%e\n", g[i].dopb);
	}
	fclose(out);
}

void write_gridding(inputPars *par, struct grid *g){
	write_grid_ascii(par, g);
	write_density_ascii(par, g);
	write_temperature_ascii(par, g);
	write_nmol_ascii(par, g);
	write_velocity_ascii(par, g);
	write_doppler_ascii(par, g);
}

void
write_levelpopulation_ascii(inputPars *par, struct grid *g, molData *m){
	FILE *out;
	char out_filename[255];
	int i,l;
	sprintf(out_filename, "%s/Output/levelpopulation_ascii.out", modelfolder);
	out = fopen(out_filename, "w");
	for(i = 0; i < par->ncell-par->sinkPoints; i++){
		for(l = 0; l < m[0].nlev; l++){
			fprintf(out, "%e ", g[i].mol[0].pops[l]);
		}
		fprintf(out, "\n");
	}
	fclose(out);
}

void
read_grid_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	int i,k;
	sprintf(out_filename, "%s/Output/grid_ascii.out", modelfolder);
	// TODO: Segmentation fault occurs after finalizing the terminal window (here and in all the following read routines!):
	if((out = fopen(out_filename, "r")) == NULL) bail_out("ERROR: No file 'grid_ascii.out' found.\n");
	for(i = 0; i < par->ncell; i++){
		fscanf(out, "%le %le %le %d %d ", &g[i].x[0], &g[i].x[1], &g[i].x[2], &g[i].sink, &g[i].numNeigh);

		g[i].neigh=malloc(sizeof(int)*g[i].numNeigh);
		for(k = 0; k < g[i].numNeigh; k++){
			fscanf(out, "%d ", &g[i].neigh[k]);
		}
		fscanf(out, "\n");

		g[i].id=i;
		g[i].dopb=0.;
	}
	fclose(out);
}

void
read_density_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	sprintf(out_filename, "%s/Output/density_ascii.out", modelfolder);
	int i;
	if((out = fopen(out_filename, "r")) == NULL) bail_out("ERROR: No file 'density_ascii.out' found.\n");
	for(i = 0; i < par->ncell; i++){
		fscanf(out, "%le\n", &g[i].dens[0]);
	}
	fclose(out);
}

void
read_temperature_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	sprintf(out_filename, "%s/Output/temperature0_ascii.out", modelfolder);
	int i;
	if((out = fopen(out_filename, "r")) == NULL) bail_out("ERROR: No file 'temperature0_ascii.out' found.\n");
	for(i = 0; i < par->ncell; i++){
		fscanf(out, "%le\n", &g[i].t[0]);
	}
	fclose(out);
	sprintf(out_filename, "%s/Output/temperature1_ascii.out", modelfolder);
	if((out = fopen(out_filename, "r")) == NULL) bail_out("ERROR: No file 'temperature1_ascii.out' found.\n");
	for(i = 0; i < par->ncell; i++){
		fscanf(out, "%le\n", &g[i].t[1]);
	}
	fclose(out);
}

void
read_nmol_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	sprintf(out_filename, "%s/Output/nmol_ascii.out", modelfolder);
	int i;
	if((out = fopen(out_filename, "r")) == NULL) bail_out("ERROR: No file 'nmol_ascii.out' found.\n");
	for(i = 0; i < par->ncell; i++){
		fscanf(out, "%le\n", &g[i].nmol[0]);
	}
	fclose(out);
}

/*
void
read_velocity_ascii(inputPars *par, struct grid *g){
	FILE *out;
	int i;
	double vel[3];
	out = fopen("velocity_ascii.out", "w");
	for(i = 0; i < par->ncell; i++){
		velocity(g[i].x[0],g[i].x[1],g[i].x[2], vel);
		fscanf(out, "%le %le %le\n", &vel[0], &vel[1], &vel[2]);
	}
	fclose(out);
}*/

void
read_doppler_ascii(inputPars *par, struct grid *g){
	FILE *out;
	char out_filename[255];
	sprintf(out_filename, "%s/Output/doppler_ascii.out", modelfolder);
	int i;
	if((out = fopen(out_filename, "r")) == NULL) bail_out("ERROR: No file 'doppler_ascii.out' found.\n");
	for(i = 0; i < par->ncell; i++){
		fscanf(out, "%le\n", &g[i].dopb);
	}
	fclose(out);
}

void 
read_gridding(inputPars *par, struct grid *g){
	
	// TODO: either use old values, which are read-in or use new assignment via model.c routines here:
	read_grid_ascii(par, g);
	read_density_ascii(par, g);
	read_temperature_ascii(par, g);
	read_nmol_ascii(par, g);
	//write_velocity_ascii(par, g);
	read_doppler_ascii(par, g);
	
	// assign new model values to old grid point distribution:
	int i,j;
	double *abun;
	
	abun = malloc(sizeof(double) * par->nSpecies);
	for(i = 0; i < par->pIntensity; i++){
		density(g[i].x[0],g[i].x[1],g[i].x[2],g[i].dens);
		temperature(g[i].x[0],g[i].x[1],g[i].x[2],g[i].t);
		doppler(g[i].x[0],g[i].x[1],g[i].x[2],&g[i].dopb);	
		abundance(g[i].x[0],g[i].x[1],g[i].x[2],abun);
		for(j=0; j<par->nSpecies; j++) g[i].nmol[j] = abun[j] * g[i].dens[0];
	}
}

void
read_levelpopulation_ascii(inputPars *par, struct grid *g, molData *m){
	FILE *out;
	char out_filename[255];
	int i,l;
	sprintf(out_filename, "%s/Output/levelpopulation_ascii.out", modelfolder);
	if((out = fopen(out_filename, "r")) == NULL) bail_out("ERROR: No file './levelpopulation_ascii.out' found.\n");
	// TODO: in general m[0] = m[ipart]
	for(i = 0; i < par->ncell-par->sinkPoints; i++){
		for(l = 0; l < m[0].nlev; l++){
			fscanf(out, "%le ", &g[i].mol[0].pops[l]);
		}
		fscanf(out, "\n");
	}
	fclose(out);
}

void
set_levelpopulation_ascii(inputPars *par, struct grid *g, molData *m){
	int i,l;
	// TODO: in general m[0] = m[ipart]
	for(i = 0; i < par->ncell-par->sinkPoints; i++){
		for(l = 0; l < m[0].nlev; l++){
			if(l==0) g[i].mol[0].pops[l] = nb;
			if(l==2) g[i].mol[0].pops[l] = n0;
			if(l==1 || l==3) g[i].mol[0].pops[l] = npm;
		}
	}
}
