/*
 *  main.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  Created by Christian Brinch on 16/11/06.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@strw.leidenuniv.nl>
 *  Sterrewacht Leiden
 *  Leiden University
 *	All rights reserved.
 *
 *  LIME is derived from RATRAN by Michiel Hogerheijde and Floris van der Tak,
 *  Copyright 2000, Hogerheijde and van der Tak, A&A, 362, 697, 2000.
 *
 *  DISCLAIMER: LIME is provided as is and the author does not resume any 
 *  responsibility for erroneous results due to bugs in the code.
 *
 *  Any publication with results obtain using LIME should refer to 
 *  Brinch & Hogerheijde, A&A, 523, A25, 2010 
 *
 */

#include "lime.h"
#if LINEPOLARIZATION == ON
#include "linepolarization.h"
int **dirs;
double ***inidirs;
#endif

int main(int argc, char *argv[]) {

	int i;					
	int initime  = time(0);
	int popsdone = 0;
	molData		*m;			
	inputPars	par;
	struct grid	*g;
	image		*img;

	par.dust       = NULL;
	par.inputfile  = NULL;
	par.outputfile = NULL;
	par.gridfile   = NULL;
	par.pregrid	   = NULL;
	
	// ****************************************************
	// **     User-defined command line parameter        **
	// ****************************************************
	//
	// Default values:
	//
	dogridding                 = 1;
	dolevelpopulations         = 1;
	doraytracing               = 1;
	setlevelpopulations        = 0;
	nb                         = -1;
	n0                         = -1;
	npm                        = -1;
	settransitionprobabilities = 0;
	A                          = -1;
	AforB                      = -1;
	C                          = -1;
	ininphot                   = 100;
	uservar1                   = -1;
	uservar2                   = -1;
	uservar3                   = -1;
	uservar4                   = -1;
	
	//
	// Command line values:
	//
	for(i = 1; i < argc; i++){
		if(!strcmp(argv[i], "--dogridding"))                 dogridding                 = atoi(argv[i+1]);
		if(!strcmp(argv[i], "--dolevelpopulations"))         dolevelpopulations         = atoi(argv[i+1]);
		if(!strcmp(argv[i], "--doraytracing"))               doraytracing               = atoi(argv[i+1]);
		if(!strcmp(argv[i], "--setlevelpopulations"))        setlevelpopulations        = atoi(argv[i+1]);
		if(!strcmp(argv[i], "--nb"))                         nb                         = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--n0"))                         n0                         = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--npm"))                        npm                        = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--settransitionprobabilities")) settransitionprobabilities = atoi(argv[i+1]);
		if(!strcmp(argv[i], "--A"))                          A                          = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--AforB"))                      AforB                      = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--C"))                          C                          = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--ininphot"))                   ininphot                   = atoi(argv[i+1]);
		if(!strcmp(argv[i], "--uservar1"))                   uservar1                   = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--uservar2"))                   uservar2                   = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--uservar3"))                   uservar3                   = strtod(argv[i+1], 0);
		if(!strcmp(argv[i], "--uservar4"))                   uservar4                   = strtod(argv[i+1], 0);
	}

	if(!silent) greetings();
	if(!silent) screenInfo();
	parseInput(&par,&img,&m);
	gridAlloc(&par,&g);

	if(dogridding){
		if(par.pregrid) predefinedGrid(&par,g);
		else buildGrid(&par,g);
		write_gridding(&par, g);
	}
	else{
		read_gridding(&par, g);

		distCalc(&par, g);
		const gsl_rng *ran = gsl_rng_alloc(gsl_rng_ranlxs2);	/* Random number generator */
		gsl_rng_set(ran,time(0));
		getArea(&par,g, ran);
		getVelosplines(&par,g);
	}

	if(loud) report(1,&par,g);
	
#if LINEPOLARIZATION == ON
	dirs = malloc(par.ncell * sizeof(int));
	inidirs = malloc(par.ncell * sizeof(double));
#endif
	
	for(i=0;i<par.nImages;i++){
		if(dolevelpopulations){
			if(img[i].doline==1 && popsdone==0) {
				levelPops(m,&par,g);
				popsdone=1;
			}
			write_levelpopulation_ascii(&par, g, m);
		}
		else{
			int id;
			for(id=0;id<par.ncell;id++) {
			  g[id].mol=malloc(sizeof(struct populations)*par.nSpecies);
			}
			/* Read in all molecular data */
			for(id=0;id<par.nSpecies;id++) molinit(m,&par,g,id);

			if(doraytracing && !setlevelpopulations) read_levelpopulation_ascii(&par, g, m);
			if(doraytracing &&  setlevelpopulations)  set_levelpopulation_ascii(&par, g, m);
		}
		if(img[i].doline==0) {
		  continuumSetup(i,img,m,&par,g);
		}
		if(doraytracing){
			raytrace(i,&par,g,m,img);
			writefits(i,m,img);
		}
	}

	if(!silent) goodnight(initime,img[0].filename);
	return 0;
}
