/*
 *  molinit.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  Created by Christian Brinch on 16/11/06.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@strw.leidenuniv.nl>
 *  Sterrewacht Leiden
 *  Leiden University 
 *	All rights reserved.
 *
 */

#include "lime.h"
#if LINEPOLARIZATION == ON
#include "linepolarization.h"
#endif

void
kappa(molData *m, struct grid *g, inputPars *par, int s){
	FILE *fp;
	char string[80];
	int i=0,k,iline,id;
	double loglam, *lamtab, *kaptab, *kappatab;
	gsl_spline *spline;
	gsl_interp_accel *acc=gsl_interp_accel_alloc();

	kappatab   	   = malloc(sizeof(double)*m[s].nline);
	m[s].cmb	   = malloc(sizeof(double)*m[s].nline);
	m[s].local_cmb = malloc(sizeof(double)*m[s].nline);
	
	if(par->dust == NULL){
		for(i=0;i<m[s].nline;i++) kappatab[i]=0.;
	} else {
		if((fp=fopen(par->dust, "r"))==NULL){
			if(!silent) bail_out("Error opening dust opacity data file!");
			exit(1);
		}
		while(fgetc(fp) != EOF){
			fgets(string,80,fp);
			i++;
		}
		rewind(fp);
		lamtab=malloc(sizeof(double)*i);
		kaptab=malloc(sizeof(double)*i);
		for(k=0;k<i;k++){
			fscanf(fp,"%lf %lf\n", &lamtab[k], &kaptab[k]);
			lamtab[k]=log10(lamtab[k]/1e6);
			kaptab[k]=log10(kaptab[k]);
		}
		fclose(fp);
		spline=gsl_spline_alloc(gsl_interp_cspline,i);
		gsl_spline_init(spline,lamtab,kaptab,i);
		for(i=0;i<m[s].nline;i++) {
			loglam=log10(CLIGHT/m[s].freq[i]);
			kappatab[i]=0.1*pow(10.,gsl_spline_eval(spline,loglam,acc));
		}
		gsl_interp_accel_free(acc);
		gsl_spline_free(spline);	
		free(kaptab);
		free(lamtab);
	}
	for(iline=0;iline<m[s].nline;iline++){
		for(id=0;id<par->ncell;id++){
			g[id].mol[s].knu[iline]=kappatab[iline]*2.4*AMU/gasIIdust*g[id].dens[0];

			/* Check if input model supplies a dust temperature. Otherwise use the kinetic temperature */
			if(g[id].t[1]==-1) {
			  g[id].mol[s].dust[iline]=planckfunc(iline,g[id].t[0],m,s);
			} else {
			  g[id].mol[s].dust[iline]=planckfunc(iline,g[id].t[1],m,s);
			}
		}
		// fix the normalization at 230GHz
		m[s].norm=planckfunc(0,par->tcmb,m,0);
		m[s].norminv=1./m[s].norm;
		if(par->tcmb>0.) m[s].cmb[iline]=planckfunc(iline,par->tcmb,m,s)/m[s].norm;
		else m[s].cmb[iline]=0.;		
		m[s].local_cmb[iline]=planckfunc(iline,2.728,m,s)/m[s].norm;
		
	}
	free(kappatab);
	return;
}


double
planckfunc(int iline, double temp, molData *m,int s){
	double bb;
	if(temp<eps) bb = 0.0;
	else {
		if (HPLANCK*m[s].freq[iline]>100.*KBOLTZ*temp) 
			bb=2.*HPLANCK*pow((m[s].freq[iline]/CLIGHT),2)*m[s].freq[iline]*exp(-HPLANCK*m[s].freq[iline]/KBOLTZ/temp);
 		else 
			bb=2.*HPLANCK*pow((m[s].freq[iline]/CLIGHT),2)*m[s].freq[iline]/(exp(HPLANCK*m[s].freq[iline]/KBOLTZ/temp)-1);
	}
	return bb;
}


void
molinit(molData *m, inputPars *par, struct grid *g, int i){

	int id, ilev, iline, itrans, itemp, *ntemp, tnint=-1, idummy, ipart;
	double fac, uprate, downrate=0, dummy, amass;
	struct data { double *colld, *temp; } *part;

	char string[80], specref[80];
	FILE *fp;
	
	if((fp=fopen(par->moldatfile[i], "r"))==NULL) {
		if(!silent) bail_out("Error opening molecular data file");
		exit(1);
	} 
	
	fgets(string, 80, fp);
	fgets(specref, 80, fp);	
	fgets(string, 80, fp);
	fscanf(fp, "%lf\n", &amass);
	fgets(string, 80, fp);
	fscanf(fp, "%d\n", &m[i].nlev);	
	fgets(string, 80, fp);

#if LINEPOLARIZATION == ON

	if(!silent) warning("Line Polarization is switched on.");

    // TODO-GENERALIZE: in general the magnetic field strength is local, splitting of levels has then to be calculated locally!
	double energy_SI2cgs = 1.0 / HPLANCK / CLIGHT / 100.0;
	double B[3];
	int cell = 0;
	magneticfield(g[cell].x[0], g[cell].x[1], g[cell].x[2], B);
	double absB;
	absB = sqrt(B[0]*B[0] + B[1]*B[1] + B[2]*B[2]); // in Tesla = 10^4 Gauss
	absB = 1e-10; // 1e-10 Tesla = 1 microGauss

	double *temp_eterm;
	double *temp_gstat;
	int *temp_J;

	int J, M, nlevmag, ilevmag, nlinemag;
	double Energy;
	int nlevunmag;

	nlevmag  = 0;
	ilevmag  = 0;
	nlinemag = 0;

	temp_eterm = malloc(sizeof(double)*m[i].nlev);
	temp_gstat = malloc(sizeof(double)*m[i].nlev);
	temp_J     = malloc(sizeof(int)*m[i].nlev);
	for(ilev=0;ilev<m[i].nlev;ilev++){
		fscanf(fp, "%d %lf %lf %d", &idummy, &temp_eterm[ilev], &temp_gstat[ilev], &temp_J[ilev]);
		fgets(string, 80, fp);
	}

	// Determine number of energy levels including the magnetic sub-levels defined by the user,
	// split each energy level into 2J+1 magnetic sub-levels with M_J = -J,-J+1,...,J-1,J:
	nlevmag = m[i].nlev;
	nlevunmag = m[i].nlev;
	
	for(ilev = 0; ilev < nlevunmag; ilev++){
		J = temp_J[ilev];
		if(linepolarization_AreMagneticSublevelsTreatedExplicitly(J)){
			nlevmag += 2*J+1;
			nlevmag--; // M=0 is already considered in m[i].nlev
		}
	}

	if(DebugLinePolarization) fprintf(DebugLinePolarizationFile, "-- New energy levels due to magnetic splitting --\n");
	if(DebugLinePolarization) fprintf(DebugLinePolarizationFile, "nlev    = %d\n", m[i].nlev);
	if(DebugLinePolarization) fprintf(DebugLinePolarizationFile, "nlevmag = %d\n", nlevmag);

	// adjust number of energy levels:
	m[i].eterm = malloc(sizeof(double)*nlevmag);
	m[i].gstat = malloc(sizeof(double)*nlevmag);
	m[i].J     = malloc(sizeof(double)*nlevmag);
	m[i].M     = malloc(sizeof(double)*nlevmag);

	// write new energy levels including the magnetic substates:
	ilevmag = 0;
	for(ilev = 0; ilev < m[i].nlev; ilev++){

		Energy = temp_eterm[ilev];
		J = temp_J[ilev];

		if(DebugLinePolarization) fprintf(DebugLinePolarizationFile, "J = %3d\n", J);

		// split each energy level into 2J+1 magnetic substates with M_J = -J,-J+1,...,J-1,J:
		if(linepolarization_AreMagneticSublevelsTreatedExplicitly(J)){
			for(M = -J; M <= J; M++){
				m[i].J[ilevmag] = J;
				m[i].M[ilevmag] = M;
				// TODO-GENERALIZE:
				// In general, the energy shift depends on the local magnetic field strength!
				// Therefore, the energy shift has to be computed during the ray-tracing such as the frequency shift due to the velocity field.
				// Numerical issue to have in mind: To define two levels with the same energy results in a singular matrix.
				// BEWARE: In LIME, eterm is defined in cm^-1, NOT in SI units! Therefore apply operator SI-to-cgs = / HPLANCK / CLIGHT / 100.
				m[i].eterm[ilevmag] = Energy + M * Lande(J, 0 , J) * BOHRMAG * absB * energy_SI2cgs;
				m[i].gstat[ilevmag] = 1.0; // i.e. no degeneracy any more
				ilevmag++;
				if(DebugLinePolarization) fprintf(DebugLinePolarizationFile, " M = %3d\n", M);
			}
		}
		else{
			m[i].J[ilevmag] = J;
			m[i].M[ilevmag] = 0;
			m[i].eterm[ilevmag] = Energy;
			m[i].gstat[ilevmag] = 2.*J+1.;
			ilevmag++;
		}
	}
	m[i].nlev = nlevmag;

	free(temp_eterm);
	free(temp_gstat);
	free(temp_J);

	// Check summary:
	if(DebugLinePolarization){
		fprintf(DebugLinePolarizationFile, "\nSummary - New energy levels:\n");
		for(ilev=0;ilev<m[i].nlev;ilev++){
			fprintf(DebugLinePolarizationFile, "ilev = %3d\n", ilev);
			fprintf(DebugLinePolarizationFile, " J = %3d\n", m[i].J[ilev]);
			fprintf(DebugLinePolarizationFile, "  M = %3d\n", m[i].M[ilev]);
		}
	}

#else

	m[i].eterm=malloc(sizeof(double)*m[i].nlev);
	m[i].gstat=malloc(sizeof(double)*m[i].nlev);

	for(ilev=0;ilev<m[i].nlev;ilev++){
		fscanf(fp, "%d %lf %lf", &idummy, &m[i].eterm[ilev], &m[i].gstat[ilev]);
		fgets(string, 80, fp);
	}

#endif

	fgets(string, 80, fp);
	fscanf(fp, "%d\n", &m[i].nline);	
	fgets(string, 80, fp);

#if LINEPOLARIZATION == ON

	int *temp_lau;
	int *temp_lal;
	double *temp_aeinst;
	double *temp_freq;
	int Jup, Jdown, Mup, Mdown;

	temp_lau    = malloc(sizeof(int)*m[i].nline);
	temp_lal    = malloc(sizeof(int)*m[i].nline);
	temp_aeinst = malloc(sizeof(double)*m[i].nline);
	temp_freq   = malloc(sizeof(double)*m[i].nline);

	for(iline=0;iline<m[i].nline;iline++){
		fscanf(fp, "%d %d %d %lf %lf %lf\n", &idummy, &temp_lau[iline], &temp_lal[iline], &temp_aeinst[iline], &temp_freq[iline], &dummy);
		temp_freq[iline] *= 1e+9; // convert into SI units (Hz instead of GHz)
		temp_lau[iline]-=1; // convert from level number to Jup
		temp_lal[iline]-=1; // convert from level number to Jdown
	}

	if(DebugLinePolarization) fprintf(DebugLinePolarizationFile, "\n-- New radiative transitions due to magnetic splitting --\n");

	// adjust number of radiative transitions of the molecule, according to quantum rules:
	nlinemag = m[i].nline;
	for(iline = 0; iline < m[i].nline; iline++){
		Jup   = temp_lau[iline];
		Jdown = temp_lal[iline];
		if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jup) && linepolarization_AreMagneticSublevelsTreatedExplicitly(Jdown)){
			for(Mup = -Jup; Mup <= +Jup; Mup++){
				for(Mdown = -Jdown; Mdown <= +Jdown; Mdown++){
					if(abs(Mdown - Mup) <= 1){
						nlinemag ++;
					}
				}
			}
			nlinemag--; // transition Mup=0 --> Mdown=0 is already considered in m[i].nline
		}
		else if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jup)){
			Mdown=0;
			for(Mup = -Jup; Mup <= +Jup; Mup++){
				if(abs(Mdown - Mup) <= 1){
					nlinemag ++;
				}
			}
			nlinemag--; // transition Mup=0 --> Mdown=0 is already considered in m[i].nline
		}
		else if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jdown)){
			Mup=0;
			for(Mdown = -Jdown; Mdown <= +Jdown; Mdown++){
				if(abs(Mdown - Mup) <= 1){
					nlinemag++;
				}
			}
			nlinemag--; // transition Mup=0 --> Mdown=0 is already considered in m[i].nline
		}
	}
	if(DebugLinePolarization){
		fprintf(DebugLinePolarizationFile, "nline =    %3d\n", m[i].nline);
		fprintf(DebugLinePolarizationFile, "nlinemag = %3d\n", nlinemag);
	}

	// compute properties (J, J', M, M', AEinstein, Frequency) of the radiative transitions:
	m[i].lau       = malloc(sizeof(int)*nlinemag);
	m[i].lal       = malloc(sizeof(int)*nlinemag);
	m[i].aeinst    = malloc(sizeof(double)*nlinemag);
	m[i].freq      = malloc(sizeof(double)*nlinemag);
	m[i].beinstu   = malloc(sizeof(double)*nlinemag);
	m[i].beinstl   = malloc(sizeof(double)*nlinemag);
	m[i].phot      = malloc(sizeof(double)*nlinemag*max_phot);
	m[i].photpx    = malloc(sizeof(double)*nlinemag*max_phot);
	m[i].photpy    = malloc(sizeof(double)*nlinemag*max_phot);
	m[i].photpz    = malloc(sizeof(double)*nlinemag*max_phot);
	m[i].phot_para = malloc(sizeof(double)*nlinemag*max_phot);
	m[i].phot_perp = malloc(sizeof(double)*nlinemag*max_phot);
	m[i].ds        = malloc(sizeof(double)*max_phot);
	m[i].vfac      = malloc(sizeof(double)*max_phot);
	m[i].weight    = malloc(sizeof(double)*max_phot);
	m[i].jbar      = malloc(sizeof(double)*nlinemag);
	m[i].jbarpx    = malloc(sizeof(double)*nlinemag);
	m[i].jbarpy    = malloc(sizeof(double)*nlinemag);
	m[i].jbarpz    = malloc(sizeof(double)*nlinemag);
	m[i].jbar_para = malloc(sizeof(double)*nlinemag);
	m[i].jbar_perp = malloc(sizeof(double)*nlinemag);
	m[i].jbar_dM0  = malloc(sizeof(double)*nlinemag);
	m[i].jbar_dM1  = malloc(sizeof(double)*nlinemag);
	m[i].mau       = malloc(sizeof(int)*nlinemag);
	m[i].mal       = malloc(sizeof(int)*nlinemag);

	int ilinemag;
	double AEinstein, Frequency;
	double dEup, dEdown;
	double CGmean;
	int nCGmean;
	// TODO: Define the appropriate way for computing the CG coefficients in cases with multiple splitted and non-splitted lines
	int CGtreatment = 1;
	
	ilinemag = 0;
	for(iline = 0; iline < m[i].nline; iline++){
		Jup       = temp_lau[iline];
		Jdown     = temp_lal[iline];
		Frequency = temp_freq[iline];
		AEinstein = temp_aeinst[iline];
		if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jup) && linepolarization_AreMagneticSublevelsTreatedExplicitly(Jdown)){
			for(Mup = -Jup; Mup <= Jup; Mup++){
				for(Mdown = -Jdown; Mdown <= Jdown; Mdown++){
					if(abs(Mup-Mdown) <= 1){
						m[i].lau[ilinemag] = Jup;
						m[i].lal[ilinemag] = Jdown;
						m[i].mau[ilinemag] = Mup;
						m[i].mal[ilinemag] = Mdown;
						
						m[i].aeinst[ilinemag]  = AEinstein;
						m[i].aeinst[ilinemag] *= pow(linepolarization_GetClebschGordonCoefficient(Jup, Mup, Mdown), 2);
						
						// TODO-GENERALIZE:
						// In general, the frequency shift depends on the local magnetic field strength!
						// Therefore, the frequency shift has to be computed during the ray-tracing such as the frequency shift due to the velocity field.
						// Only in this special test case: B = absB = 1e-10 = const.;
						m[i].freq[ilinemag] = Frequency;
						dEup   = Mup   * Lande(Jup  , 0 , Jup  ) * BOHRMAG * absB;
						dEdown = Mdown * Lande(Jdown, 0 , Jdown) * BOHRMAG * absB;
						m[i].freq[ilinemag] += (dEup - dEdown) / HPLANCK;
						ilinemag++;
					}
				}
			}
		}
		else if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jup)){
			Mdown = 0;
			for(Mup = -Jup; Mup <= Jup; Mup++){
				if(abs(Mup-Mdown) <= 1){
					m[i].lau[ilinemag] = Jup;
					m[i].lal[ilinemag] = Jdown;
					m[i].mau[ilinemag] = Mup;
					m[i].mal[ilinemag] = Mdown;
					
					// TODO: Define the appropriate way for computing the CG coefficients in cases with multiple splitted and non-splitted lines
					m[i].aeinst[ilinemag] = AEinstein;
					switch(CGtreatment){
						case 1:
							// No difference in transition probability
							break;
						case 2:
							//Mdown = 0;
							m[i].aeinst[ilinemag] *= pow(linepolarization_GetClebschGordonCoefficient(Jup, Mup, Mdown), 2);
							break;
						case 3:
							CGmean = 0;
							nCGmean = 0;
							for(Mdown = -Jdown; Mdown <= Jdown; Mdown++){
								if(abs(Mup-Mdown) <= 1){
									CGmean += pow(linepolarization_GetClebschGordonCoefficient(Jup, Mup, Mdown), 2);
									nCGmean++;
								}
							}
							CGmean /= nCGmean;
							m[i].aeinst[ilinemag] *= CGmean;
							break;
					}
					
					m[i].freq[ilinemag] = Frequency;
					dEup   = Mup   * Lande(Jup  , 0 , Jup  ) * BOHRMAG * absB;
					dEdown = 0;
					m[i].freq[ilinemag] += (dEup - dEdown) / HPLANCK;
					ilinemag++;
				}
			}
		}
		else if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jdown)){
			Mup = 0;
			for(Mdown = -Jdown; Mdown <= Jdown; Mdown++){
				if(abs(Mup-Mdown) <= 1){
					m[i].lau[ilinemag] = Jup;
					m[i].lal[ilinemag] = Jdown;
					m[i].mau[ilinemag] = Mup;
					m[i].mal[ilinemag] = Mdown;
					
					m[i].aeinst[ilinemag] = AEinstein;
					switch(CGtreatment){
						case 1:
							break;
						case 2:
							Mup = 0;
							m[i].aeinst[ilinemag] *= pow(linepolarization_GetClebschGordonCoefficient(Jup, Mup, Mdown), 2);
							break;
						case 3:
							CGmean = 0;
							nCGmean = 0;
							for(Mup = -Jup; Mup <= Jup; Mup++){
								if(abs(Mup-Mdown) <= 1){
									CGmean += pow(linepolarization_GetClebschGordonCoefficient(Jup, Mup, Mdown), 2);
									nCGmean++;
								}
							}
							CGmean /= nCGmean;
							m[i].aeinst[ilinemag] *= CGmean;
							break;
					}
					
					m[i].freq[ilinemag] = Frequency;
					dEup   = 0;
					dEdown = Mdown * Lande(Jdown, 0 , Jdown) * BOHRMAG * absB;
					m[i].freq[ilinemag] += (dEup - dEdown) / HPLANCK;
					ilinemag++;
				}
			}
		}
		else{
			m[i].lau[ilinemag] = Jup;
			m[i].lal[ilinemag] = Jdown;
			m[i].aeinst[ilinemag] = AEinstein;
			m[i].freq[ilinemag] = Frequency;
			m[i].mau[ilinemag] = 0;
			m[i].mal[ilinemag] = 0;
			ilinemag++;
		}

        // According to Deguchi and Watson (1984), Eq. (13) and following:
		if(Jup-Jdown > 1) m[i].aeinst[ilinemag-1] = 0;
	}

	m[i].nline = nlinemag;

	free(temp_lau);
	free(temp_lal);
	free(temp_aeinst);
	free(temp_freq);

	// Check summary:
	if(DebugLinePolarization){
		fprintf(DebugLinePolarizationFile, "\nSummary - New radiative transitions:\n");
		for(iline=0;iline<m[i].nline;iline++){
			fprintf(DebugLinePolarizationFile, "iline = %3d\n", iline);
			fprintf(DebugLinePolarizationFile, " Jup =   %3d\n", m[i].lau[iline]);
			fprintf(DebugLinePolarizationFile, " Jdown = %3d\n", m[i].lal[iline]);
			fprintf(DebugLinePolarizationFile, " Mup =   %3d\n", m[i].mau[iline]);
			fprintf(DebugLinePolarizationFile, " Mdown = %3d\n", m[i].mal[iline]);
		}
	}
	
	//
	// Only used for test runs / setting a uniform Einstein coefficient manually:
	//
	if(settransitionprobabilities && AforB == -1){
		for(iline = 0; iline < m[i].nline; iline++){
			m[i].aeinst[iline] = A; // in [s^-1]
		}
	}

#else

	m[i].lal		= malloc(sizeof(int)*m[i].nline);
	m[i].lau		= malloc(sizeof(int)*m[i].nline);
	m[i].aeinst		= malloc(sizeof(double)*m[i].nline);
	m[i].freq		= malloc(sizeof(double)*m[i].nline);
	m[i].beinstu	= malloc(sizeof(double)*m[i].nline);
	m[i].beinstl	= malloc(sizeof(double)*m[i].nline);
    m[i].phot       = malloc(sizeof(double)*m->nline*max_phot);
    m[i].ds         = malloc(sizeof(double)*max_phot);
    m[i].vfac       = malloc(sizeof(double)*max_phot);
    m[i].weight     = malloc(sizeof(double)*max_phot);
    m[i].jbar       = malloc(sizeof(double)*m[i].nline);

	for(iline=0;iline<m[i].nline;iline++){
		fscanf(fp, "%d %d %d %lf %lf %lf\n", &idummy, &m[i].lau[iline], &m[i].lal[iline], &m[i].aeinst[iline], &m[i].freq[iline], &dummy);
		m[i].freq[iline]*=1e9;
		m[i].lau[iline]-=1;
		m[i].lal[iline]-=1;
	}

#endif

	for(iline=0;iline<m[i].nline;iline++){
/*		m[i].freq[iline]=(m[i].eterm[m[i].lau[iline]]-m[i].eterm[m[i].lal[iline]])*100*CLIGHT; */
		m[i].beinstu[iline]=m[i].aeinst[iline]*(CLIGHT/m[i].freq[iline])*(CLIGHT/m[i].freq[iline])/(HPLANCK*m[i].freq[iline])/2.;
		m[i].beinstl[iline]=m[i].gstat[m[i].lau[iline]]/m[i].gstat[m[i].lal[iline]]*m[i].beinstu[iline];
	}
	amass*=AMU;
	for(id=0;id<par->ncell;id++) {
		g[id].mol[i].dopb=sqrt(g[id].dopb*g[id].dopb+2.*KBOLTZ/amass*g[id].t[0]);
		g[id].mol[i].binv=1./g[id].mol[i].dopb;
	}
	
	//
	// Only used for test runs / setting an uniform Einstein coefficient manually:
	//
	if(settransitionprobabilities && AforB != -1){
		for(iline = 0; iline < m[i].nline; iline++){
			m[i].aeinst[iline]  = A; // in [s^-1]
			m[i].beinstu[iline] = AforB*(CLIGHT/m[i].freq[iline])*(CLIGHT/m[i].freq[iline])/(HPLANCK*m[i].freq[iline])/2.; // AforB in [s^-1]
			m[i].beinstl[iline] = m[i].gstat[m[i].lau[iline]]/m[i].gstat[m[i].lal[iline]]*m[i].beinstu[iline];
		}
	}

#if LINEPOLARIZATION == ON
	int ntransmag;
	int *temp_lcl;
	int *temp_lcu;
	double *temp_colld;
#endif

	if(par->lte_only==0){
		fgets(string, 80, fp);
		fscanf(fp,"%d\n", &m[i].npart);
#if LINEPOLARIZATION == ON
		if(DebugLinePolarization){
			fprintf(DebugLinePolarizationFile, "\nStart collisions:\n");
			fprintf(DebugLinePolarizationFile, "m[i].npart = %d\n", m[i].npart);
		}
#endif

		/* collision partner sanity check */
		if(m[i].npart > par->collPart){
		  m[i].npart = par->collPart;
		}
		if(m[i].npart < par->collPart){
		  if(!silent) bail_out("Error: Too many density profiles defined");
		  exit(1);
		}

		m[i].ntrans = malloc(sizeof(int)*m[i].npart);
		ntemp = malloc(sizeof(int)*m[i].npart);
		part = malloc(sizeof(struct data) * m[i].npart);
	
	
		for(ipart=0;ipart<m[i].npart;ipart++){	  
			fgets(string, 80, fp);
			fgets(string, 200, fp);
			fgets(string, 80, fp);
			fscanf(fp,"%d\n", &m[i].ntrans[ipart]);
			fgets(string, 80, fp);
			fscanf(fp,"%d\n", &ntemp[ipart]);
			fgets(string, 80, fp);

			part[ipart].temp=malloc(sizeof(double)*ntemp[ipart]);


			if(ipart==0){
#if LINEPOLARIZATION == ON
				temp_lcl = malloc(sizeof(int)*m[i].ntrans[ipart]);
				temp_lcu = malloc(sizeof(int)*m[i].ntrans[ipart]);
#else
				m[i].lcl = malloc(sizeof(int)*m[i].ntrans[ipart]);
				m[i].lcu = malloc(sizeof(int)*m[i].ntrans[ipart]);
#endif
			}
	
			for(itemp=0;itemp<ntemp[ipart];itemp++){
				fscanf(fp, "%lf", &part[ipart].temp[itemp]);
			}
			fscanf(fp,"\n");
			fgets(string, 80, fp);


#if LINEPOLARIZATION == ON
			temp_colld = malloc(sizeof(double)*m[i].ntrans[ipart]*ntemp[ipart]);
#else
			part[ipart].colld=malloc(sizeof(double)*m[i].ntrans[ipart]*ntemp[ipart]);
#endif

			// Reading collision rates:
			for(itrans=0;itrans<m[i].ntrans[ipart];itrans++){
#if LINEPOLARIZATION == ON
				fscanf(fp, "%d %d %d", &idummy, &temp_lcu[itrans], &temp_lcl[itrans]);
				temp_lcu[itrans]-=1;
				temp_lcl[itrans]-=1;
				for(itemp=0;itemp<ntemp[ipart];itemp++){
					fscanf(fp, "%lf", &temp_colld[itrans*ntemp[ipart]+itemp]);
					temp_colld[itrans*ntemp[ipart]+itemp]/=1.e6;
				}
#else
				fscanf(fp, "%d %d %d", &idummy, &m[i].lcu[itrans], &m[i].lcl[itrans]);
				m[i].lcu[itrans]-=1;
				m[i].lcl[itrans]-=1;
				for(itemp=0;itemp<ntemp[ipart];itemp++){
					fscanf(fp, "%lf", &part[ipart].colld[itrans*ntemp[ipart]+itemp]);
					part[ipart].colld[itrans*ntemp[ipart]+itemp]/=1.e6;
				}
#endif

				fscanf(fp,"\n");
			}

#if LINEPOLARIZATION == ON
			// compute new number of collisional transitions (e.g. only up to Jup = 42 in 'co@rovib.dat' case)
			ntransmag = m[i].ntrans[ipart];
			for(itrans = 0; itrans < m[i].ntrans[ipart]; itrans++){
				Jup   = temp_lcu[itrans];
				Jdown = temp_lcl[itrans];
				if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jup))
					ntransmag += 2*Jup; // = 2J+1 -1 (M=0 --> M'=0 already considered in m[i].ntrans[ipart])
				if(linepolarization_AreMagneticSublevelsTreatedExplicitly(Jdown))
					ntransmag += 2*Jdown; // = 2J+1 -1 (M=0 --> M'=0 already considered in m[i].ntrans[ipart])
			}
			if(DebugLinePolarization){
				fprintf(DebugLinePolarizationFile, "m[i].ntrans[ipart] = %d\n", m[i].ntrans[ipart]);
				fprintf(DebugLinePolarizationFile, "ntransmag          = %d\n", ntransmag);
			}

			// set new transition probabilities
			m[i].lcl = malloc(sizeof(int)*ntransmag);
			m[i].lcu = malloc(sizeof(int)*ntransmag);
			m[i].mcl = malloc(sizeof(int)*ntransmag);
			m[i].mcu = malloc(sizeof(int)*ntransmag);
			part[ipart].colld = malloc(sizeof(double)*ntransmag*ntemp[ipart]);

			int itransmag = 0;
			int Mupmin, Mdownmin;
			for(itrans = 0; itrans < m[i].ntrans[ipart]; itrans++){

				Jup   = temp_lcu[itrans];
				Jdown = temp_lcl[itrans];
				
				if(!linepolarization_AreMagneticSublevelsTreatedExplicitly(Jdown)){
					Mdownmin = 0;
				}
				else{
					Mdownmin = Jdown;
				}
				if(!linepolarization_AreMagneticSublevelsTreatedExplicitly(Jup)){
					Mupmin = 0;
				}
				else{
					Mupmin = Jup;
				}

				for(Mup = -Mupmin; Mup <= Mupmin; Mup++){
					for(Mdown = -Mdownmin; Mdown <= Mdownmin; Mdown++){
						m[i].lcu[itransmag] = Jup;
						m[i].lcl[itransmag] = Jdown;
						m[i].mcu[itransmag] = Mup;
						m[i].mcl[itransmag] = Mdown;
						for(itemp = 0; itemp < ntemp[ipart]; itemp++){
							part[ipart].colld[itransmag*ntemp[ipart]+itemp] = temp_colld[itrans*ntemp[ipart]+itemp];
							//
							// Only used for test runs / setting an uniform collision rate manually:
							//
							if(settransitionprobabilities) part[ipart].colld[itransmag*ntemp[ipart]+itemp] = C / 1e+6; // in [m^3 s^-1]
						}
						itransmag++;
					}
				}

			}
			if(DebugLinePolarization){
				fprintf(DebugLinePolarizationFile, "final itransmag    = %d\n", itransmag);
			}

			m[i].ntrans[ipart] = ntransmag;
#endif
		} // for(ipart=0;ipart<m[i].npart;ipart++){
		fclose(fp);


		for(id=0;id<par->ncell;id++){
			g[id].mol[i].partner=malloc(sizeof(struct rates)*m[i].npart);
			for(ipart=0;ipart<m[i].npart;ipart++){
				g[id].mol[i].partner[ipart].up = malloc(sizeof(double)*m[i].ntrans[ipart]);
				g[id].mol[i].partner[ipart].down = malloc(sizeof(double)*m[i].ntrans[ipart]);
			}
		}

		for(id=0;id<par->ncell;id++){
			if(g[id].nmol[i]>eps){
				for(ipart=0;ipart<m[i].npart;ipart++){
					for(itrans=0;itrans<m[i].ntrans[ipart];itrans++){
					 	if((g[id].t[0]>part[ipart].temp[0])&&(g[id].t[0]<part[ipart].temp[ntemp[ipart]-1])){
							for(itemp=0;itemp<ntemp[ipart]-1;itemp++){
								if((g[id].t[0]>part[ipart].temp[itemp])&&(g[id].t[0]<=part[ipart].temp[itemp+1])){
									tnint=itemp;
								}
							}
							fac=(g[id].t[0]-part[ipart].temp[tnint])/(part[ipart].temp[tnint+1]-part[ipart].temp[tnint]);
							downrate=part[ipart].colld[itrans*ntemp[ipart]+tnint]+fac*(part[ipart].colld[itrans*ntemp[ipart]+tnint+1]-part[ipart].colld[itrans*ntemp[ipart]+tnint]);
						}
					 	else{
							if(g[id].t[0]<=part[ipart].temp[0]) downrate=part[ipart].colld[itrans*ntemp[ipart]];
							if(g[id].t[0]>=part[ipart].temp[ntemp[ipart]-1]) downrate=part[ipart].colld[itrans*ntemp[ipart]+ntemp[ipart]-1];
						}
						int k,l;
#if LINEPOLARIZATION == ON
						Jup   = m[i].lcu[itrans];
						Jdown = m[i].lcl[itrans];
						Mup   = m[i].mcu[itrans];
						Mdown = m[i].mcl[itrans];
						k = linepolarization_GetEnergyLevelNumber(  Jup,   Mup, m, i);
						l = linepolarization_GetEnergyLevelNumber(Jdown, Mdown, m, i);					
#else
						k = m[i].lcu[itrans];
						l = m[i].lcl[itrans];
#endif
						uprate =
								m[i].gstat[k]
								/
								m[i].gstat[l]
								*
								downrate
								*
								exp(
									-HCKB
									*
									(m[i].eterm[k] - m[i].eterm[l])
									/
									g[id].t[0]
							    );
						g[id].mol[i].partner[ipart].up[itrans]=uprate;
						g[id].mol[i].partner[ipart].down[itrans]=downrate;
					}
				}
			}
		}
#if LINEPOLARIZATION == ON
		// Write database file as summary for testing and debugging:
		// TODO: add correct names of molecule instead of 'xxx'
		FILE *moldatfilemag;
		char moldatfilemagname[255];
		strcpy(moldatfilemagname, par->moldatfile[i]);
		strcat(moldatfilemagname, ".out");
		moldatfilemag = fopen(moldatfilemagname, "w");
		
		fprintf(moldatfilemag, "!MOLECULE\n");
		fprintf(moldatfilemag, "xxx\n");
		fprintf(moldatfilemag, "!MOLECULAR WEIGHT\n");
		fprintf(moldatfilemag, "%.1f\n", amass/AMU);
		fprintf(moldatfilemag, "!NUMBER OF ENERGY LEVELS\n");
		fprintf(moldatfilemag, "%d\n", nlevmag);
		fprintf(moldatfilemag, "!LEVEL + ENERGIES(cm^-1) + WEIGHT + J + M + DELTAENERGIES(cm^-1)\n");
		for(ilevmag = 0; ilevmag < nlevmag; ilevmag++){
			if(linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].J[ilevmag]))
				fprintf(moldatfilemag, "%5d      %e %8.1f %4d %+3d    %+e\n", ilevmag+1, m[i].eterm[ilevmag], m[i].gstat[ilevmag], m[i].J[ilevmag], m[i].M[ilevmag], m[i].M[ilevmag] * Lande(m[i].J[ilevmag],0,m[i].J[ilevmag]) * BOHRMAG * absB / HPLANCK / CLIGHT / 100.);
			else
				fprintf(moldatfilemag, "%5d      %e %8.1f %4d\n", ilevmag+1, m[i].eterm[ilevmag], m[i].gstat[ilevmag], m[i].J[ilevmag]);
		}
		fprintf(moldatfilemag, "!NUMBER OF RADIATIVE TRANSITIONS\n");
		fprintf(moldatfilemag, "%d\n", nlinemag);
		fprintf(moldatfilemag, "!TRANS + UP + MUP + LOW + MLOW + EINSTEINA(s^-1) + FREQ(GHz) + E_u(K) + EINSTEINBUP(s g^-1) + EINSTEINBLOW(s g^-1)\n");
		for(ilinemag = 0; ilinemag < nlinemag; ilinemag++){
			if(linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lau[ilinemag]) && linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lal[ilinemag]))
				fprintf(moldatfilemag, "%5d %5d %+5d %5d %+6d     %.3e       %f %7.2f     %.3e           %.3e\n", ilinemag+1, m[i].lau[ilinemag]+1, m[i].mau[ilinemag], m[i].lal[ilinemag]+1, m[i].mal[ilinemag], m[i].aeinst[ilinemag], m[i].freq[ilinemag]*1e-9, HPLANCK*m[i].freq[ilinemag]/KBOLTZ, m[i].beinstu[ilinemag], m[i].beinstl[ilinemag]);
			if(linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lau[ilinemag]) && !linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lal[ilinemag]))
				fprintf(moldatfilemag, "%5d %5d %+5d %5d            %.3e       %f %7.2f     %.3e           %.3e\n", ilinemag+1, m[i].lau[ilinemag]+1, m[i].mau[ilinemag], m[i].lal[ilinemag]+1, m[i].aeinst[ilinemag], m[i].freq[ilinemag]*1e-9, HPLANCK*m[i].freq[ilinemag]/KBOLTZ, m[i].beinstu[ilinemag], m[i].beinstl[ilinemag]);
			if(!linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lau[ilinemag]) && linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lal[ilinemag]))
				fprintf(moldatfilemag, "%5d %5d       %5d %+6d     %.3e       %f %7.2f     %.3e           %.3e\n", ilinemag+1, m[i].lau[ilinemag]+1, m[i].lal[ilinemag]+1, m[i].mal[ilinemag], m[i].aeinst[ilinemag], m[i].freq[ilinemag]*1e-9, HPLANCK*m[i].freq[ilinemag]/KBOLTZ, m[i].beinstu[ilinemag], m[i].beinstl[ilinemag]);
			if(!linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lau[ilinemag]) && !linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lal[ilinemag]))
				fprintf(moldatfilemag, "%5d %5d       %5d            %.3e       %f %7.2f     %.3e           %.3e\n", ilinemag+1, m[i].lau[ilinemag]+1, m[i].lal[ilinemag]+1, m[i].aeinst[ilinemag], m[i].freq[ilinemag]*1e-9, HPLANCK*m[i].freq[ilinemag]/KBOLTZ, m[i].beinstu[ilinemag], m[i].beinstl[ilinemag]);
		}
		fprintf(moldatfilemag, "!NUMBER OF COLL PARTNERS\n");
		fprintf(moldatfilemag, "%d\n", m[i].npart);
		for(ipart = 0; ipart < m[i].npart; ipart++){
			fprintf(moldatfilemag, "!COLLISIONS BETWEEN\n");
			fprintf(moldatfilemag, "1 xxx - H2\n");
			fprintf(moldatfilemag, "!NUMBER OF COLL TRANS\n");
			fprintf(moldatfilemag, "%d\n", m[i].ntrans[ipart]);
			fprintf(moldatfilemag, "!NUMBER OF COLL TEMPS\n");
			fprintf(moldatfilemag, "%d\n", ntemp[ipart]);
			fprintf(moldatfilemag, "!COLL TEMPS\n");
			for(itemp = 0; itemp < ntemp[ipart]; itemp++){
				fprintf(moldatfilemag, "%7.1f ", part[ipart].temp[itemp]);
			}
			fprintf(moldatfilemag, "\n");
			fprintf(moldatfilemag, "!TRANS + UP + MUP + LOW + MLOW + COLLRATES(cm^3 s^-1)\n");
			for(itrans = 0; itrans < m[i].ntrans[ipart]; itrans++){
				if(linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lcu[itrans]) && linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lcl[itrans]))
					fprintf(moldatfilemag, "%5d %5d %+5d %5d  %+5d    ", itrans+1, m[i].lcu[itrans]+1, m[i].mcu[itrans], m[i].lcl[itrans]+1, m[i].mcl[itrans]);
				if(linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lcu[itrans]) && !linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lcl[itrans]))
					fprintf(moldatfilemag, "%5d %5d %+5d %5d            ", itrans+1, m[i].lcu[itrans]+1, m[i].mcu[itrans], m[i].lcl[itrans]+1);
				if(!linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lcu[itrans]) && linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lcl[itrans]))
					fprintf(moldatfilemag, "%5d %5d       %5d  %+5d    ", itrans+1, m[i].lcu[itrans]+1, m[i].lcl[itrans]+1, m[i].mcl[itrans]);
				if(!linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lcu[itrans]) && !linepolarization_AreMagneticSublevelsTreatedExplicitly(m[i].lcl[itrans]))
					fprintf(moldatfilemag, "%5d %5d       %5d           ", itrans+1, m[i].lcu[itrans]+1, m[i].lcl[itrans]+1);
				for(itemp = 0; itemp < ntemp[ipart]; itemp++){
					fprintf(moldatfilemag, "%.1e ", part[ipart].colld[itrans*ntemp[ipart]+itemp] *1e+6);
				}
				fprintf(moldatfilemag, "\n");
			}
		}
		fprintf(moldatfilemag, "!Notes:\n");
		fprintf(moldatfilemag, "!This file is automatically produced by LIME, if (and only if) line polarization is used.\n");
		fprintf(moldatfilemag, "!It provides an overview of useful information about the run performed.\n");
		fprintf(moldatfilemag, "!BEWARE: This file is NOT an appropriate moldat input file for LIME and is NOT written in the official Leiden database format!\n");
		fclose(moldatfilemag);
#endif
		free(part);
	}


	for(id=0;id<par->ncell; id++){
		g[id].mol[i].pops = malloc(sizeof(double)*m[i].nlev);
		g[id].mol[i].dust = malloc(sizeof(double)*m[i].nline);
		g[id].mol[i].knu  = malloc(sizeof(double)*m[i].nline);
		for(ilev=0;ilev<m[i].nlev;ilev++) g[id].mol[i].pops[ilev]=0.0;
	}
	kappa(m,g,par,i);
}

