/*
 *  photon.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  Created by Christian Brinch on 15/11/06.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@strw.leidenuniv.nl>
 *  Sterrewacht Leiden
 *  Leiden University 
 *  All rights reserved.
 *
 */


int count = 0;

#include "lime.h"
#if LINEPOLARIZATION == ON
#include "linepolarization.h"
#endif

#if inidirflag == 1
int
sortangles(int id, int dir, int newid, struct grid *g, const gsl_rng *ran) {
	int i,j,itmp,val;
	/* int b,l,k; */
	double *angle,tmp,best;
	int *n;
	
	angle=malloc(sizeof(double)*g[newid].numNeigh);	
	n=malloc(sizeof(int)*g[newid].numNeigh);	
	
	best=1e30;
	for(i=0;i<g[newid].numNeigh;i++){
		angle[i]=( g[id].dir[dir].xn[0]*g[newid].dir[i].xn[0]
				  +g[id].dir[dir].xn[1]*g[newid].dir[i].xn[1]
				  +g[id].dir[dir].xn[2]*g[newid].dir[i].xn[2]);
		n[i]=i;
		/*	  if(angle[i]<best){
		 best=angle[i];
		 b=i;
		 }*/
	}
	
	for(i=g[newid].numNeigh-1; i>=0; i--) {
		for(j=1; j<=i; j++) {
			if(angle[j-1]<angle[j]) {
				tmp=angle[j-1];
				angle[j-1]=angle[j];
				angle[j]=tmp;
				
				itmp=n[j-1];
				n[j-1]=n[j];
				n[j]=itmp;
			}
		}	
	}	  
	
  	if(gsl_rng_uniform(ran)<1./((1-angle[0])/(1-angle[1])+1) ) {
		val=n[0];
	} else {
		val=n[1];	
	}
	free(angle);
	free(n);
	return val;
	
	/*    return angle[b]; */
}
#elif inidirflag == 2
int
sortangles(double *inidir, int id, struct grid *g, const gsl_rng *ran) {
	int i,n[2];
	double angle,exitdir[2];
	
	exitdir[0]=1e30;
    exitdir[1]=1e31;
	n[0]=-1;
    n[1]=-1;
	for(i=0;i<g[id].numNeigh;i++){
		angle=( inidir[0]*g[id].dir[i].xn[0]
			   +inidir[1]*g[id].dir[i].xn[1]
			   +inidir[2]*g[id].dir[i].xn[2]);
		if(angle<exitdir[0]){
			exitdir[1]=exitdir[0];
			n[1]=n[0];
			exitdir[0]=angle;
			n[0]=i;
		} else if(angle<exitdir[1]) {
			exitdir[1]=angle;
			n[1]=i;
		}
	}
    if(gsl_rng_uniform(ran)<1./((1-exitdir[0])/(1-exitdir[1])+1) ) {
		if(n[0]==-1){
			if(!silent) bail_out("Photon propagation error");
			exit(1);
		}
		return n[0];
	} else {
		if(n[1]==-1){
			if(!silent) bail_out("Photon propagation error");
			exit(1);
		}
		return n[1];
	}
}
#endif

void
velocityspline(struct grid *g, int id, int k, double binv, double deltav, double *vfac){
	int nspline,ispline,naver,iaver;
	double v1,v2,s1,s2,sd,v,vfacsub,d;
	
	v1=deltav-veloproject(g[id].dir[k].xn,g[id].vel);
	v2=deltav-veloproject(g[id].dir[k].xn,g[g[id].neigh[k]].vel);
	
	nspline=(fabs(v1-v2)*binv < 1) ? 1 : (int)(fabs(v1-v2)*binv);
	*vfac=0.;
	s2=0;
	v2=v1;
	
	for(ispline=0;ispline<nspline;ispline++){
		s1=s2;
		s2=((double)(ispline+1))/(double)nspline;					
		v1=v2;
		d=s2*g[id].ds[k];
		v2=deltav-(g[id].a4[k]*pow(d,4)+g[id].a3[k]*pow(d,3)+g[id].a2[k]*pow(d,2)+g[id].a1[k]*d+g[id].a0[k]);
		naver=(1 > fabs(v1-v2)*binv) ? 1 : (int)(fabs(v1-v2)*binv);
		for(iaver=0;iaver<naver;iaver++){
			sd=s1+(s2-s1)*((double)iaver-0.5)/(double)naver;
			d=sd*g[id].ds[k];
			v=deltav-(g[id].a4[k]*pow(d,4)+g[id].a3[k]*pow(d,3)+g[id].a2[k]*pow(d,2)+g[id].a1[k]*d+g[id].a0[k]);
			vfacsub=gaussline(v,binv);
			*vfac+=vfacsub/(double)naver;
		}
	}
	*vfac= *vfac/(double)nspline;
	return;
}


void
velocityspline_lin(struct grid *g, int id, int k, double binv, double deltav, double *vfac){
	int nspline,ispline,naver,iaver;
	double v1,v2,s1,s2,sd,v,vfacsub,d;
	
	v1=deltav-veloproject(g[id].dir[k].xn,g[id].vel);
	v2=deltav-veloproject(g[id].dir[k].xn,g[g[id].neigh[k]].vel);
	
	nspline=(fabs(v1-v2)*binv < 1) ? 1 : (int)(fabs(v1-v2)*binv);
	*vfac=0.;
	s2=0;
	v2=v1;
	
	for(ispline=0;ispline<nspline;ispline++){
		s1=s2;
		s2=((double)(ispline+1))/(double)nspline;					
		v1=v2;
		d=s2*g[id].ds[k];
		v2=deltav-(g[id].a1[k]*d+g[id].a0[k]);
		naver=(1 > fabs(v1-v2)*binv) ? 1 : (int)(fabs(v1-v2)*binv);
		for(iaver=0;iaver<naver;iaver++){
			sd=s1+(s2-s1)*((double)iaver-0.5)/(double)naver;
			d=sd*g[id].ds[k];
			v=deltav-(g[id].a1[k]*d+g[id].a0[k]);
			vfacsub=gaussline(v,binv);
			*vfac+=vfacsub/(double)naver;
		}
	}
	*vfac= *vfac/(double)nspline;
	return;
}



void
velocityspline2(double x[3], double dx[3], double ds, double binv, double deltav, double *vfac){
	int nspline,ispline,naver,iaver;
	double v1,v2,s1,s2,sd,v,vfacsub,vel[3];
	
	velocity(x[0],x[1],x[2],vel);
	v1=deltav-veloproject(dx,vel);
	velocity(x[0]+(dx[0]*ds),x[1]+(dx[1]*ds),x[2]+(dx[2]*ds),vel);
	v2=deltav-veloproject(dx,vel);
	
	nspline=(fabs(v1-v2)*binv < 1) ? 1 : (int)(fabs(v1-v2)*binv);
	*vfac=0.;
	s2=0;
	v2=v1;
	
	for(ispline=0;ispline<nspline;ispline++){
		s1=s2;
		s2=((double)(ispline+1))/(double)nspline;					
		v1=v2;
		velocity(x[0]+(s2*dx[0]*ds),x[1]+(s2*dx[1]*ds),x[2]+(s2*dx[2]*ds),vel);
		v2=deltav-veloproject(dx,vel);
		naver=(1 > fabs(v1-v2)*binv) ? 1 : (int)(fabs(v1-v2)*binv);
		for(iaver=0;iaver<naver;iaver++){
			sd=s1+(s2-s1)*((double)iaver-0.5)/(double)naver;
			velocity(x[0]+(sd*dx[0]*ds),x[1]+(sd*dx[1]*ds),x[2]+(sd*dx[2]*ds),vel);
			v=deltav-veloproject(dx,vel);
			vfacsub=gaussline(v,binv);
			*vfac+=vfacsub/(double)naver;
		}
	}
	*vfac= *vfac/(double)nspline;
	
	return;
}


double veloproject(double dx[3], double *vel)
{
	return dx[0]*vel[0]+dx[1]*vel[1]+dx[2]*vel[2];
}


double gaussline(double v, double sigma)
{
	int maxgau=101,maxsig=4,ival;
	double fac,val;
	
	fac=(maxgau-1)/maxsig;
	ival=(int)(fac*(fabs(v)*sigma))+1;
	if((ival-1)>=maxgau) return 0.;
	val=(ival*ival)/(fac*fac);
	return exp(-val);
}


void
photon(int id, struct grid *g, molData *m, int iter, const gsl_rng *ran,inputPars *par,blend *matrix){
	
	int iphot,iline,jline,here,there,firststep,dir,np_per_line,ip_at_line,l,c;
#if inidirflag == 1
	int inidir;
#elif inidirflag == 2
	double inidir[3];
	
	dirs[id] = malloc(g[id].nphot * sizeof(int));
	inidirs[id] = malloc(g[id].nphot * sizeof(double));
	for(iphot=0;iphot<g[id].nphot;iphot++){
		inidirs[id][iphot] = malloc(3 * sizeof(double));
	}
#endif
	int *counta, *countb,nlinetot;
	double deltav,segment,vblend,snu,dtau,jnu,alpha,ds,vfac[par->nSpecies];
	double *tau,vel[3],x[3];
#if LINEPOLARIZATION == ON
	double jnu_para, jnu_perp, alpha_para, alpha_perp;
	double snu_para, snu_perp, dtau_para, dtau_perp;
	double angle_nB, n[3];
	double *taupx, *taupy, *taupz;
	double *tau_para, *tau_perp;
#endif
	
	lineCount(par->nSpecies, m, &counta, &countb, &nlinetot);	
	
	tau=malloc(sizeof(double)*nlinetot);
#if LINEPOLARIZATION == ON
	taupx=malloc(sizeof(double)*nlinetot);
	taupy=malloc(sizeof(double)*nlinetot);
	taupz=malloc(sizeof(double)*nlinetot);
	
	tau_para = malloc(sizeof(double)*nlinetot);
	tau_perp = malloc(sizeof(double)*nlinetot);
#endif
	
	velocity(g[id].x[0],g[id].x[1],g[id].x[2],vel);
	
	for(iphot=0;iphot<g[id].nphot;iphot++){
		firststep=1;
		
		for(iline=0;iline<nlinetot;iline++){
			m[0].phot[iline+iphot*m[0].nline]=0.;
			tau[iline]=0.;
#if LINEPOLARIZATION == ON
			m[0].photpx[iline+iphot*m[0].nline]=0;
			m[0].photpy[iline+iphot*m[0].nline]=0;
			m[0].photpz[iline+iphot*m[0].nline]=0;
			taupx[iline]=0;
			taupy[iline]=0;
			taupz[iline]=0;

			m[0].phot_para[iline+iphot*m[0].nline]=0;
			m[0].phot_perp[iline+iphot*m[0].nline]=0;
			tau_para[iline]=0;
			tau_perp[iline]=0;
#endif
		}
		
		/* Initial velocity, direction and frequency offset  */		
#if inidirflag == 1
		inidir=iphot%g[id].numNeigh;
		
		iter=(int) (gsl_rng_uniform(ran)*3.);
		np_per_line=(int) g[id].nphot/g[id].numNeigh;
		ip_at_line=(int) iphot/g[id].numNeigh;
		segment=1/(2.*np_per_line)*(2*ip_at_line-np_per_line+iter);
		deltav=segment*4.3*g[id].dopb+veloproject(g[id].dir[inidir].xn,vel);
		
		dir=inidir;
		here=g[id].id;
		there=g[g[here].neigh[dir]].id;

#elif inidirflag == 2
		// new inidir routine from Christian's developers version:
		
		//
		// From code documentation of the gnu scientific library (gsl):
		//
		//  Function: double gsl_rng_uniform (const gsl_rng * r)
		//   This function returns a double precision floating point number uniformly distributed in the range [0,1). 
		//   The range includes 0.0 but excludes 1.0.
		//
		
		// equal treatment of all three directions:
		inidir[0] = 2.0 * gsl_rng_uniform(ran) - 1.0;
		inidir[1] = 2.0 * gsl_rng_uniform(ran) - 1.0;
		inidir[2] = 2.0 * gsl_rng_uniform(ran) - 1.0;
		vectoralgebra_Normalize(inidir);
		
		// Store directions for later use in the routine getjbar() below:
		inidirs[id][iphot][0] = inidir[0];
		inidirs[id][iphot][1] = inidir[1];		
		inidirs[id][iphot][2] = inidir[2];
				 
		iter=(int) (gsl_rng_uniform(ran)*3.);
		np_per_line=(int) g[id].nphot/g[id].numNeigh;
		ip_at_line=(int) iphot/g[id].numNeigh;
		segment=1/(2.*np_per_line)*(2*ip_at_line-np_per_line+iter);
				 
		dir=sortangles(inidir,id,g,ran);
		dirs[id][iphot] = dir;
		here=g[id].id;
		there=g[g[here].neigh[dir]].id;
		deltav=segment*4.3*g[id].dopb+veloproject(g[id].dir[dir].xn,vel);
#endif	
		
		// use the initial outgoing dir as the direction of all photons along the diffuse path:
		
		/* Photon propagation loop */
		/*  loops over 'dir' as well */
		do{
			// use the actual dir as the direction of the local photons along the diffuse path:
#if LINEPOLARIZATION == ON
			angle_nB = linepolarization_GetAngleBetweenPropagationDirectionAndMagneticField(g, here, inidir);
			n[0] = inidir[0];
			n[1] = inidir[1];
			n[2] = inidir[2];
#endif
			
			if(firststep){
				firststep=0;				
				ds=g[here].ds[dir]/2.;
				for(l=0;l<par->nSpecies;l++){
					if(!par->pregrid) velocityspline(g,here,dir,g[id].mol[l].binv,deltav,&vfac[l]);
					else velocityspline_lin(g,here,dir,g[id].mol[l].binv,deltav,&vfac[l]);
					m[l].vfac[iphot]=vfac[0];
#if inidirflag == 1
					m[l].weight[iphot]=g[here].w[inidir];
#elif inidirflag == 2
					m[l].weight[iphot]=1;
#endif
				}
#if inidirflag == 1
				for(l=0;l<3;l++) x[l]=g[here].x[l]+(g[here].dir[dir].xn[l] * g[id].ds[inidir]/2.);
#elif inidirflag == 2
				for(l=0;l<3;l++) x[l]=g[here].x[l]+(g[here].dir[dir].xn[l] * g[id].ds[dir]/2.);
#endif
			} else {
				ds=g[here].ds[dir];
				for(l=0;l<3;l++) x[l]=g[here].x[l];
			}
			
			for(l=0;l<par->nSpecies;l++){
				if(!par->pregrid) velocityspline(g,here,dir,g[id].mol[l].binv,deltav,&vfac[l]);
				else velocityspline_lin(g,here,dir,g[id].mol[l].binv,deltav,&vfac[l]);
			}
			
			for(iline=0;iline<nlinetot;iline++){
				jnu=0.;
				alpha=0.;
				sourceFunc_line(&jnu,&alpha,m,vfac[counta[iline]],g,here,counta[iline],countb[iline]);
				sourceFunc_cont(&jnu,&alpha,g,here,counta[iline],countb[iline]);
				
#if LINEPOLARIZATION == ON
				jnu_para   = 0;
				jnu_perp   = 0;
				alpha_para = 0;
				alpha_perp = 0;
							
				linepolarization_sourceFunc_line(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, m, vfac[counta[iline]], g, here, counta[iline], countb[iline],  angle_nB);
				linepolarization_sourceFunc_cont(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, g, here, counta[iline], countb[iline], m);
#endif
				
				if(par->blend){
					for(c = 0; c < nblendedlines; c++){
						if(matrix[c].line1 == iline){
							jline = matrix[c].line2;
							if(!par->pregrid) velocityspline(g,here,dir,g[id].mol[counta[jline]].binv,deltav-matrix[jline].deltav,&vblend);
							else velocityspline_lin(g,here,dir,g[id].mol[counta[jline]].binv,deltav-matrix[jline].deltav,&vblend);	
							sourceFunc_line(&jnu,&alpha,m,vblend,g,here,counta[jline],countb[jline]);
							
#if LINEPOLARIZATION == ON
							linepolarization_sourceFunc_line(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, m, vblend, g, here, counta[jline], countb[jline], angle_nB);
#endif
						} // if(matrix[c].line1 == iline || matrix[c].line2 == iline)
					} // for(c=0;c<sizeof(matrix)/sizeof(blend);c++){
				} // if(par->blend)
				
				// Otherwise, the old values from the last iline will be used again!
				if(fabs(alpha)>0.){
					snu=(jnu/alpha)*m[0].norminv;
					dtau=alpha*ds;
				}
				m[0].phot[iline+iphot*m[0].nline]+=exp(-tau[iline])*(1.-exp(-dtau))*snu;
				tau[iline]+=dtau;
				
#if LINEPOLARIZATION == ON
				if(fabs(alpha_para)>0.){
					snu_para  = (jnu_para/alpha_para) * m[0].norminv;
					dtau_para = alpha_para * ds;
				}
				else{
					snu_para  = 0;
					dtau_para = 0;
				}
				if(fabs(alpha_perp)>0.){
					snu_perp  = (jnu_perp/alpha_perp) * m[0].norminv;
					dtau_perp = alpha_perp * ds;
				}
				else{
					snu_perp  = 0;
					dtau_perp = 0;
				}
                
				double I_para, I_perp; // Polarization Intensities parallel and perpendicular to Magnetic Field at location "here"
				double Icartesian[3];
				
				// get Polarization Intensities parallel and perpendicular to Magnetic Field at location "here":
				I_para = exp(-tau_para[iline]) * (1.-exp(-dtau_para)) * snu_para;
				I_perp = exp(-tau_perp[iline]) * (1.-exp(-dtau_perp)) * snu_perp;
				
				// transform Polarization Intensities into a general Cartesian Frame:
				linepolarization_PolarizationFrame2CartesianFrame(g, here, n, I_para, I_perp, Icartesian);
				// transform Polarization Intensities back into the Frame parallel and perpendicular to Magnetic Field at location "id":
				linepolarization_CartesianFrame2PolarizationFrame(g, id,   n, Icartesian, &I_para, &I_perp);
				
				// add up Polarization Intensities parallel and perpendicular to Magnetic Field at location "id":
				m[0].phot_para[iline+iphot*m[0].nline] += I_para;
				m[0].phot_perp[iline+iphot*m[0].nline] += I_perp;
				
				
				// transform Polarization Intensities into a general Cartesian Frame:
				linepolarization_PolarizationFrame2CartesianFrame(g, here, n, dtau_para, dtau_perp, Icartesian);
				// transform Polarization Intensities back into the Frame parallel and perpendicular to Magnetic Field at location "id":
				linepolarization_CartesianFrame2PolarizationFrame(g, id,   n, Icartesian, &dtau_para, &dtau_perp);
				
				// add up optical depths of Intensities parallel and perpendicular to Magnetic Field at location "id":
				tau_para[iline] += dtau_para;
				tau_perp[iline] += dtau_perp;
#endif
				
			} // for(iline=0;iline<nlinetot;iline++)
			
#if inidirflag == 1
			dir=sortangles(id,inidir,there,g,ran);
#elif inidirflag == 2
			dir=sortangles(inidir,there,g,ran);
#endif
			
			here=there;
			there=g[g[here].neigh[dir]].id;
		} while(!g[there].sink);
		
		/* Add cmb contribution */
		if(m[0].cmb[0]>0.){
#if LINEPOLARIZATION == ON
			double fx, fy, fz;
			
			// get the propagation direction of the light from the sink to id
			n[0] = g[id].x[0] - g[there].x[0];
			n[1] = g[id].x[1] - g[there].x[1];
			n[2] = g[id].x[2] - g[there].x[2];
						
			n[0] = fabs(n[0]);
			n[1] = fabs(n[1]);
			n[2] = fabs(n[2]);
			
			fx = (n[1]+n[2])/(n[0]+n[1]+n[2])/2.;
			fy = (n[0]+n[2])/(n[0]+n[1]+n[2])/2.;
			fz = (n[0]+n[1])/(n[0]+n[1]+n[2])/2.;
#endif
			for(iline=0;iline<nlinetot;iline++){
				m[0].phot[iline+iphot*m[0].nline]   += exp(-tau[iline])   * m[counta[iline]].cmb[countb[iline]];
#if LINEPOLARIZATION == ON
				m[0].photpx[iline+iphot*m[0].nline] += fx * exp(-taupx[iline]) * m[counta[iline]].cmb[countb[iline]];
				m[0].photpy[iline+iphot*m[0].nline] += fy * exp(-taupy[iline]) * m[counta[iline]].cmb[countb[iline]];
				m[0].photpz[iline+iphot*m[0].nline] += fz * exp(-taupz[iline]) * m[counta[iline]].cmb[countb[iline]];
								
				m[0].phot_para[iline+iphot*m[0].nline] += 0.5 * exp(-tau_para[iline]) * m[counta[iline]].cmb[countb[iline]];
				m[0].phot_perp[iline+iphot*m[0].nline] += 0.5 * exp(-tau_perp[iline]) * m[counta[iline]].cmb[countb[iline]];
#endif
			}
		} // if(m[0].cmb[0]>0.)
	} // for(iphot=0;iphot<g[id].nphot;iphot++)
	
	free(tau);
#if LINEPOLARIZATION == ON
	free(taupx);
	free(taupy);
	free(taupz);
	free(tau_para);
	free(tau_perp);
#endif
	free(counta);
	free(countb);
	
}






double dmin(double x, double y){
	if(x < y) return x;
	else return y;
}






void getjbar(int posn, molData *m, struct grid *g, inputPars *par){
	int iline,iphot;	      
	double tau, snu, vsum=0., jnu, alpha;
	int *counta, *countb, nlinetot;
	
	lineCount(par->nSpecies, m, &counta, &countb, &nlinetot);	
	
#if LINEPOLARIZATION == ON
	double jnu_para, jnu_perp, alpha_para, alpha_perp;
	double snu_para, snu_perp, tau_para, tau_perp;
	double angle_nB, n[3];
	double jbar_para, jbar_perp;
#endif
	
	for(iline=0;iline<m[0].nline;iline++) m[0].jbar[iline]=0.;
#if LINEPOLARIZATION == ON
	for(iline=0;iline<m[0].nline;iline++){
		m[0].jbarpx[iline]=0.;
		m[0].jbarpy[iline]=0.;
		m[0].jbarpz[iline]=0.;

		m[0].jbar_para[iline]=0.;
		m[0].jbar_perp[iline]=0.;
		
		m[0].jbar_dM0[iline]=0.;
		m[0].jbar_dM1[iline]=0.;
	}
#endif
	
	for(iphot=0;iphot<g[posn].nphot;iphot++){
		if(m[0].vfac[iphot]>0){
#if LINEPOLARIZATION == ON
			// Get propagation direction of the light:
#if inidirflag == 1
			dir = iphot%g[posn].numNeigh;
			linepolarization_GetPropagationDirection_photon(n, g, posn, dir);
			angle_nB = linepolarization_GetAngleBetweenPropagationDirectionAndMagneticField(g, posn, n);
#elif inidirflag == 2
			// re-use the inidir's computed in photon():
			n[0] = inidirs[posn][iphot][0];
			n[1] = inidirs[posn][iphot][1];
			n[2] = inidirs[posn][iphot][2];
			angle_nB = linepolarization_GetAngleBetweenPropagationDirectionAndMagneticField(g, posn, n);
#endif
#endif
#if LINEPOLARIZATION == ON
			double weight = 1.0; // => I.e. the implementation of extra weight can be deleted!
#else
			double weight = 1;	
#endif
			for(iline=0;iline<m[0].nline;iline++){
				jnu=0.;
				alpha=0.;
				sourceFunc_line(&jnu,&alpha,m,m[0].vfac[iphot],g,posn,counta[iline],countb[iline]);
				sourceFunc_cont(&jnu,&alpha,g,posn,counta[iline],countb[iline]);
				if(fabs(alpha)>0.){
					snu=(jnu/alpha)*m[0].norminv;
					tau=alpha*m[0].ds[iphot];
				}
				m[0].jbar[iline]+=m[0].vfac[iphot]*(exp(-tau)*m[0].phot[iline+iphot*m[0].nline]+(1.-exp(-tau))*snu)*m[0].weight[iphot];
				
#if LINEPOLARIZATION == ON
				jnu_para = 0;
				jnu_perp = 0;
				alpha_para = 0;
				alpha_perp = 0;
				
				linepolarization_sourceFunc_line(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, m, m[0].vfac[iphot], g, posn, counta[iline], countb[iline], angle_nB);
				linepolarization_sourceFunc_cont(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, g, posn, counta[iline], countb[iline], m);
                
				if(fabs(alpha_para)>0.){
					snu_para=(jnu_para/alpha_para)*m[0].norminv;
					tau_para=alpha_para*m[0].ds[iphot];
				}
				else{
					snu_para=0;
					tau_para=0;
				}
				if(fabs(alpha_perp)>0.){
					snu_perp=(jnu_perp/alpha_perp)*m[0].norminv;
					tau_perp=alpha_perp*m[0].ds[iphot];
				}
				else{
					snu_perp=0;
					tau_perp=0;
				}
				
				// see Cortes, Crutcher & Watson (2005), eq. (15):
				jbar_para = m[0].vfac[iphot]*(exp(-tau_para)*m[0].phot_para[iline+iphot*m[0].nline]+(1.-exp(-tau_para))*snu_para)*m[0].weight[iphot];
				jbar_perp = m[0].vfac[iphot]*(exp(-tau_perp)*m[0].phot_perp[iline+iphot*m[0].nline]+(1.-exp(-tau_perp))*snu_perp)*m[0].weight[iphot];
				m[0].jbar_dM0[iline] += pow(sin(angle_nB), 2) * jbar_para * weight;
				m[0].jbar_dM1[iline] += 0.5 * (jbar_perp + pow(cos(angle_nB), 2) * jbar_para) * weight;
#endif
			}// for(iline=0;iline<m[0].nline;iline++)
#if LINEPOLARIZATION == ON
			vsum+=m[0].vfac[iphot]*m[0].weight[iphot]*weight;
#else
			vsum+=m[0].vfac[iphot]*m[0].weight[iphot];
#endif
		}
	}// for(iphot=0;iphot<g[posn].nphot;iphot++)
	
	for(iline=0;iline<m[0].nline;iline++){
		m[0].jbar[iline]=m[0].norm*m[0].jbar[iline]/vsum;
	}
	
#if LINEPOLARIZATION == ON
	for(iline=0;iline<m[0].nline;iline++){
		m[0].jbarpx[iline]    *= m[0].norm/vsum;
		m[0].jbarpy[iline]    *= m[0].norm/vsum;
		m[0].jbarpz[iline]    *= m[0].norm/vsum;
		m[0].jbar_para[iline] *= m[0].norm/vsum;
		m[0].jbar_perp[iline] *= m[0].norm/vsum;
		m[0].jbar_dM0[iline]  *= m[0].norm/vsum;
		m[0].jbar_dM1[iline]  *= m[0].norm/vsum;
	}
#endif
}

