/*
 *  raytrace.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  Created by Christian Brinch on 16/12/06.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@strw.leidenuniv.nl>
 *  Sterrewacht Leiden
 *  Leiden University
 *	All rights reserved.
 *
 */

#include "lime.h"
#if LINEPOLARIZATION == ON
#include "linepolarization.h"
#endif

void raytrace(int im, inputPars *par, struct grid *g, molData *m, image *img){
	int *counta, *countb,nlinetot;
	int ichan, posn, oposn,i,px,iline,tmptrans;
	double *tau, *subintens;
	double vfac=0.,x[3],dx[3];
	double deltav,ds,dist,ndist,size,xp,yp,zp,col,shift,minfreq,jnu,alpha,snu,dtau;
#if LINEPOLARIZATION == ON
	double jnu_para, jnu_perp, alpha_para, alpha_perp;
	double snu_para, snu_perp, dtau_para, dtau_perp;
	double *subintenspx, *subintenspy, *subintenspz;
	double *taupx, *taupy, *taupz;
	double *subintens_para, *subintens_perp;
	double *tau_para, *tau_perp;
	double snu_pol[3], dtau_pol[3];
	double angle_nB, n[3];
	
	linepolarization_GetPropagationDirection_raytrace(n, img, im);
#endif
	lineCount(par->nSpecies, m, &counta, &countb, &nlinetot);	
	if(img[im].doline==0) nlinetot=1;
	
	
	/* Fix the image parameters */
	if(img[im].freq < 0) img[im].freq=m[0].freq[img[im].trans];
	if(img[im].nchan == 0 && img[im].bandwidth>0){	 
		img[im].nchan=(int) (img[im].bandwidth/(img[im].velres/CLIGHT*img[im].freq));
	} else if (img[im].velres<0 && img[im].bandwidth>0){
		img[im].velres = img[im].bandwidth*CLIGHT/img[im].freq/img[im].nchan;
	} else img[im].bandwidth = img[im].nchan*img[im].velres/CLIGHT * img[im].freq; 
	
	if(img[im].trans<0){
		minfreq=1e30;
		tmptrans=-1;
		for(iline=0;iline<m[0].nline;iline++){
			if(fabs(img[im].freq-m[0].freq[iline])<minfreq){
				minfreq=fabs(img[im].freq-m[0].freq[iline]);
				tmptrans=iline;
			}
		}
	} else tmptrans=img[im].trans;
	
	tau = malloc(sizeof(double)*img[im].nchan);
	subintens = malloc(sizeof(double)*img[im].nchan);
#if LINEPOLARIZATION == ON
	taupx = malloc(sizeof(double)*img[im].nchan);
	taupy = malloc(sizeof(double)*img[im].nchan);
	taupz = malloc(sizeof(double)*img[im].nchan);
	subintenspx = malloc(sizeof(double)*img[im].nchan);
	subintenspy = malloc(sizeof(double)*img[im].nchan);
	subintenspz = malloc(sizeof(double)*img[im].nchan);
	tau_para = malloc(sizeof(double)*img[im].nchan);
	tau_perp = malloc(sizeof(double)*img[im].nchan);
	subintens_para = malloc(sizeof(double)*img[im].nchan);
	subintens_perp = malloc(sizeof(double)*img[im].nchan);
#endif
	
	for(px=0;px<(img[im].pxls*img[im].pxls);px++){
		for(ichan=0;ichan<img[im].nchan;ichan++){
			tau[ichan]=0.0;
			subintens[ichan]=0.0;
			img[im].pixel[px].intense[ichan]=0.0;
			img[im].pixel[px].tau[ichan]=0.0;
#if LINEPOLARIZATION == ON
			taupx[ichan] = 0.0;
			taupy[ichan] = 0.0;
			taupz[ichan] = 0.0;
			subintenspx[ichan] = 0.0;
			subintenspy[ichan] = 0.0;
			subintenspz[ichan] = 0.0;
			tau_para[ichan] = 0.0;
			tau_perp[ichan] = 0.0;
			subintens_para[ichan] = 0.0;
			subintens_perp[ichan] = 0.0;
			img[im].pixel[px].intensepx[ichan] = 0.0;
			img[im].pixel[px].intensepy[ichan] = 0.0;
			img[im].pixel[px].intensepz[ichan] = 0.0;
			img[im].pixel[px].taupx[ichan] = 0.0;
			img[im].pixel[px].taupy[ichan] = 0.0;
			img[im].pixel[px].taupz[ichan] = 0.0;
#endif
		}
		
		size=img[im].distance*img[im].imgres;
		
		xp=size*(0.5+px%img[im].pxls)-size*img[im].pxls/2.;
		yp=size*(0.5+px/img[im].pxls)-size*img[im].pxls/2.;
		zp=par->radius;
		
		/*
		 |1	  0		    0   |
		 R_x(a)=|0	cos(a)	sin(a)	|
		 |0 -sin(a)	cos(a)	|
		 
		 |cos(b)	0	-sin(b) |
		 R_y(b)=|  0		1	   0	|
		 |sin(b)	0	 cos(b) |
		 
		 |cos(b)  		    0 	   sin(b)   |
		 Rot =  |sin(a)sin(b)	cos(a)	sin(a)cos(b)|			 
		 |cos(a)sin(b)   -sin(a)  cos(a)cos(b)|
		 */
		
		
		if(sqrt(xp*xp+yp*yp)<=par->radius) {
			
 			x[0]=xp*cos(img[im].phi)                   +yp*0.                -zp*sin(img[im].phi);
			x[1]=xp*sin(img[im].theta)*sin(img[im].phi)+yp*cos(img[im].theta)+zp*sin(img[im].theta)*cos(img[im].phi);
			x[2]=xp*cos(img[im].theta)*sin(img[im].phi)-yp*sin(img[im].theta)+zp*cos(img[im].theta)*cos(img[im].phi);
			
			dx[0]= sin(img[im].phi);
			dx[1]=-sin(img[im].theta)*cos(img[im].phi);
			dx[2]=-cos(img[im].theta)*cos(img[im].phi);
			
			dist=1e30;
			posn=-1;
			
			for(i=0;i<par->ncell;i++){
				ndist=sqrt(pow(x[0]-g[i].x[0],2)+pow(x[1]-g[i].x[1],2)+pow(x[2]-g[i].x[2],2));
				if(ndist<dist){
					posn=i;
					dist=ndist;
				}
			}		
			
			ds=1e30;
			for(i=0;i<g[posn].numNeigh;i++) {
				if(g[posn].ds[i]<ds) ds=g[posn].ds[i];
			}
			col=0;
			
			do{// = "loop over position" / spatial integration
				for(ichan=0;ichan<img[im].nchan;ichan++){
					jnu=0.;
					alpha=0.;
					snu=0.;
					dtau=0.;
#if LINEPOLARIZATION == ON
					jnu_para   = 0;
					jnu_perp   = 0;
					alpha_para = 0;
					alpha_perp = 0;
					snu_para   = 0;
					snu_perp   = 0;
					dtau_para  = 0;
					dtau_perp  = 0;
					
					angle_nB = linepolarization_GetAngleBetweenPropagationDirectionAndMagneticField(g, posn, n);
#endif
					for(iline=0;iline<nlinetot;iline++){
						if(img[im].doline && m[counta[iline]].freq[countb[iline]] > img[im].freq-img[im].bandwidth/2. && m[counta[iline]].freq[countb[iline]] < img[im].freq+img[im].bandwidth/2.){
							if(img[im].trans > -1){
								shift=(m[counta[iline]].freq[countb[iline]]-m[counta[iline]].freq[img[im].trans])/m[counta[iline]].freq[img[im].trans]*CLIGHT;
							} else {
								shift=(m[counta[iline]].freq[countb[iline]]-img[im].freq)/img[im].freq*CLIGHT;
							}
							deltav=(ichan-(int)(img[im].nchan/2.))*img[im].velres-img[im].source_vel -shift;	
							
							if(!par->pregrid) velocityspline2(x,dx,ds,g[posn].mol[counta[iline]].binv,deltav,&vfac);
							else vfac=gaussline(deltav-veloproject(dx,g[posn].vel),g[posn].mol[counta[iline]].binv);			
							
							sourceFunc_line(&jnu,&alpha,m,vfac,g,posn,counta[iline],countb[iline]);
#if LINEPOLARIZATION == ON
							linepolarization_sourceFunc_line(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, m, vfac, g, posn, counta[iline], countb[iline], angle_nB);
#endif
						} // close if(img[im].doline && m[counta[iline]].freq[countb[iline]] > img[im].freq-img[im].bandwidth/2. && m[counta[iline]].freq[countb[iline]] < img[im].freq+img[im].bandwidth/2.)
					} // close for(iline=0;iline<nlinetot;iline++)
										
					if(img[im].doline && img[im].trans > -1) sourceFunc_cont(&jnu,&alpha,g,posn,0,img[im].trans);
					else if(img[im].doline && img[im].trans == -1) sourceFunc_cont(&jnu,&alpha,g,posn,0,tmptrans);
					else sourceFunc_cont(&jnu,&alpha,g,posn,0,0);
#if LINEPOLARIZATION == ON
					if(img[im].doline && img[im].trans > -1) linepolarization_sourceFunc_cont(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, g, posn, 0, img[im].trans, m);
					else if(img[im].doline && img[im].trans == -1) linepolarization_sourceFunc_cont(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, g, posn, 0, tmptrans, m);
					else linepolarization_sourceFunc_cont(&jnu_para, &jnu_perp, &alpha_para, &alpha_perp, g, posn, 0, 0, m);
#endif
					if(fabs(alpha)>0.){
						snu=(jnu/alpha)*m[0].norminv;
						dtau=alpha*ds;
					}
#if LINEPOLARIZATION == ON
					if(fabs(alpha_para)>0.){
						snu_para=(jnu_para/alpha_para)*m[0].norminv;
						dtau_para=alpha_para*ds;
					}
					if(fabs(alpha_perp)>0.){
						snu_perp=(jnu_perp/alpha_perp)*m[0].norminv;
						dtau_perp=alpha_perp*ds;
					}
#endif
					
					subintens[ichan]+=exp(-tau[ichan])*(1.-exp(-dtau))*snu;
					tau[ichan]+=dtau;
#if LINEPOLARIZATION == ON
					linepolarization_PolarizationFrame2CartesianFrame_snu_and_tau_simultaneously(g, posn, n, snu_para, snu_perp, dtau_para, dtau_perp, snu_pol, dtau_pol);

					// add to subintens:
					subintenspx[ichan]    += exp(-taupx[ichan]) * (1.-exp(-dtau_pol[0])) * snu_pol[0];
					subintenspy[ichan]    += exp(-taupy[ichan]) * (1.-exp(-dtau_pol[1])) * snu_pol[1];
					subintenspz[ichan]    += exp(-taupz[ichan]) * (1.-exp(-dtau_pol[2])) * snu_pol[2];
					subintens_para[ichan] += exp(-tau_para[ichan]) * (1.-exp(-dtau_para)) * snu_para;
					subintens_perp[ichan] += exp(-tau_perp[ichan]) * (1.-exp(-dtau_perp)) * snu_perp;
                    
					// update the total optical depth along the ray / line-of-sight:
					taupx[ichan]    += dtau_pol[0];
					taupy[ichan]    += dtau_pol[1];
					taupz[ichan]    += dtau_pol[2];
					tau_para[ichan] += dtau_para;
					tau_perp[ichan] += dtau_perp;
#endif

				}// close for(ichan=0;ichan<img[im].nchan;ichan++)
				
				
				/* new coordinates */
				for(i=0;i<3;i++) x[i]+=ds*dx[i];
				col+=ds;
				
				
				/* distance to previous closest neighbor */	
				dist=sqrt(pow(x[0]-g[posn].x[0],2)+pow(x[1]-g[posn].x[1],2)+pow(x[2]-g[posn].x[2],2));
				oposn=posn;
				
				/* find distance to oposn's neighbors and possibly assign new closest neighbor */
				for(i=0;i<g[oposn].numNeigh;i++){
					ndist=sqrt(pow(x[0]-g[g[oposn].neigh[i]].x[0],2)+pow(x[1]-g[g[oposn].neigh[i]].x[1],2)+pow(x[2]-g[g[oposn].neigh[i]].x[2],2));
					if (ndist<dist){
						posn=g[g[oposn].neigh[i]].id;
						dist=ndist;
					}
				}
				
				/* find new shortest neighbor distance  */
				ds=0.;
				for(i=0;i<g[posn].numNeigh;i++) {
					ds+=g[posn].ds[i]/g[posn].numNeigh/3.;
				}
			} while(col < 2*par->radius);
			//bail_out("STOP HERE");exit(1);
			
			/* add or subtract cmb */
			for(ichan=0;ichan<img[im].nchan;ichan++){
				if(tau[ichan]<=15) {
					subintens[ichan]+=(exp(-tau[ichan])-1.)*m[0].local_cmb[tmptrans];
				} else {
					subintens[ichan]-=m[0].local_cmb[tmptrans];
				}
#if LINEPOLARIZATION == ON
				// subtract cmb contribution:
				double fx, fy, fz;
				double nx, ny, nz;
				nx = fabs(n[0]);
				ny = fabs(n[1]);
				nz = fabs(n[2]);
				fx = 0.5*(ny+nz)/(nx+ny+nz);
				fy = 0.5*(nz+nx)/(nx+ny+nz);
				fz = 0.5*(nx+ny)/(nx+ny+nz);
				subintenspx[ichan]    += fx  * (exp(-taupx[ichan]   ) - 1.) * m[0].local_cmb[tmptrans];
				subintenspy[ichan]    += fy  * (exp(-taupy[ichan]   ) - 1.) * m[0].local_cmb[tmptrans];
				subintenspz[ichan]    += fz  * (exp(-taupz[ichan]   ) - 1.) * m[0].local_cmb[tmptrans];
				subintens_para[ichan] += 0.5 * (exp(-tau_para[ichan]) - 1.) * m[0].local_cmb[tmptrans];
				subintens_perp[ichan] += 0.5 * (exp(-tau_perp[ichan]) - 1.) * m[0].local_cmb[tmptrans];
#endif
			}
			for(ichan=0;ichan<img[im].nchan;ichan++){
				img[im].pixel[px].intense[ichan]+=subintens[ichan];
				img[im].pixel[px].tau[ichan]+=tau[ichan];

#if LINEPOLARIZATION == ON
				img[im].pixel[px].intensepx[ichan] += subintenspx[ichan];
				img[im].pixel[px].intensepy[ichan] += subintenspy[ichan];
				img[im].pixel[px].intensepz[ichan] += subintenspz[ichan];
				img[im].pixel[px].taupx[ichan]     += taupx[ichan];
				img[im].pixel[px].taupy[ichan]     += taupy[ichan];
				img[im].pixel[px].taupz[ichan]     += taupz[ichan];
#endif
			}
		} // closing of if(sqrt(xp*xp+yp*yp)<=par->radius)
		if(!silent) progressbar((double)(px)/(double)(img[im].pxls*img[im].pxls), 13);
	} // closing of loop over pixels for(px=0;px<(img[im].pxls*img[im].pxls);px++)
	
	img[im].trans=tmptrans;
	free(tau);
	free(subintens);
#if LINEPOLARIZATION == ON
	free(taupx);
	free(taupy);
	free(taupz);
	free(subintenspx);
	free(subintenspy);
	free(subintenspz);
#endif
	free(counta);
	free(countb);
}
