/*
 *  sourcefunc.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  Created by Christian Brinch on 16/11/2006
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@strw.leidenuniv.nl>
 *  Sterrewacht Leiden
 *  Leiden University 
 *	All rights reserved.
 *
 */

#include "lime.h"
#if LINEPOLARIZATION == ON
#include "linepolarization.h"
#endif

//void
//sourceFunc(double *snu, double *dtau, double ds, molData *m,double vfac,struct grid *g,int pos,int ispec, int iline, int doline){
//	double jnu, alpha; 	
//
//	/* Emission */
//	/* Continuum part:	j_nu = T_dust * kappa_nu */
//	jnu	 = g[pos].mol[ispec].dust[iline]*g[pos].mol[ispec].knu[iline];		  
//
//	/* Line part:		j_nu = v*consts*1/b*rho*n_i*A_ij */
//	if(doline) jnu	+= vfac*HPIP*g[pos].mol[ispec].binv*g[pos].nmol[ispec]*g[pos].mol[ispec].pops[m[ispec].lau[iline]]*m[ispec].aeinst[iline];
//	
//	
//
//	/* Absorption */
//	/* Continuum part: Dust opacity */
//	alpha  = g[pos].mol[ispec].knu[iline];
//
//
//	/* Line part: alpha_nu = v*const*1/b*rho*(n_j*B_ij-n_i*B_ji) */
//	if(doline) alpha += vfac*HPIP*g[pos].mol[ispec].binv*g[pos].nmol[ispec]*(g[pos].mol[ispec].pops[m[ispec].lal[iline]]*m[ispec].beinstl[iline]
//								 -g[pos].mol[ispec].pops[m[ispec].lau[iline]]*m[ispec].beinstu[iline]);
//
//
//	/* Calculate source function and tau */
//	*snu=0.;
//	*dtau=0.;
//	if(fabs(alpha)>0.){
//		*snu=(jnu/alpha)*m[ispec].norminv;
//		*dtau= alpha*ds;
//	}
//	return;
//}

void
sourceFunc_line(double *jnu, double *alpha, molData *m, double vfac, struct grid *g, int pos, int ispec, int iline){
	
	int k,l;
	
#if LINEPOLARIZATION == ON
	int Jup, Jdown, Mup, Mdown;
	Jup   = m[ispec].lau[iline];
	Jdown = m[ispec].lal[iline];
	Mup   = m[ispec].mau[iline];
	Mdown = m[ispec].mal[iline];
	k = linepolarization_GetEnergyLevelNumber(  Jup,   Mup, m, ispec);
	l = linepolarization_GetEnergyLevelNumber(Jdown, Mdown, m, ispec);
#else
	k = m[ispec].lau[iline];
	l = m[ispec].lal[iline];
	// LIME 1.03 Original:
	// ...pops[m[ispec].lau[iline]] ...
	// ...pops[m[ispec].lal[iline]] ... - ...pops[m[ispec].lau[iline]] ...
#endif
	
	/* Line part:		j_nu = v*consts*1/b*rho*n_i*A_ij */
	*jnu   += vfac*HPIP*g[pos].mol[ispec].binv*g[pos].nmol[ispec]*g[pos].mol[ispec].pops[k]*m[ispec].aeinst[iline];
	//printf("jnu = %e\n", *jnu);

	/* Line part: alpha_nu = v*const*1/b*rho*(n_j*B_ij-n_i*B_ji) */
	*alpha += vfac*HPIP*g[pos].mol[ispec].binv*g[pos].nmol[ispec]*(g[pos].mol[ispec].pops[l]*m[ispec].beinstl[iline]-g[pos].mol[ispec].pops[k]*m[ispec].beinstu[iline]);
		
	return;
}

void
sourceFunc_cont(double *jnu, double *alpha,struct grid *g,int pos,int ispec, int iline){
	
	/* Emission */
	/* Continuum part:	j_nu = T_dust * kappa_nu */
	*jnu   += g[pos].mol[ispec].dust[iline]*g[pos].mol[ispec].knu[iline];		  
	
	/* Absorption */
	/* Continuum part: Dust opacity */
	*alpha += g[pos].mol[ispec].knu[iline];
	
	return;
}
