/*
 *  writefits.c
 *  LIME, The versatile 3D line modeling environment 
 *
 *  Created by Christian Brinch on 24/04/08.
 *  Copyright 2006-2011, Christian Brinch, 
 *  <brinch@strw.leidenuniv.nl>
 *  Sterrewacht Leiden
 *  Leiden University 
 *	All rights reserved.
 *
 */

#include <cfitsio/fitsio.h>
#include "lime.h"

void 
writefits(int im, molData *m, image *img){
	double keyval;
	float *row;
	int px,py,ichan;
	fitsfile *fptr;
	int status = 0;
	int naxis=3, bitpix=-32;
	long naxes[3];
	long int fpixels[3],lpixels[3];
	char negfile[100]="! ";

	
	row = malloc(sizeof(float)*img[im].pxls);
	
	naxes[0]=img[im].pxls;
	naxes[1]=img[im].pxls;
	if(img[im].doline==1) naxes[2]=img[im].nchan;
	else if(img[im].doline==0) naxes[2]=1;

	fits_create_file(&fptr, img[im].filename, &status);
	
	if(status!=0){
		if(!silent) warning("Overwriting existing fits file                   ");
		status=0;
		strcat(negfile,img[im].filename);
		fits_create_file(&fptr, negfile, &status);
	}

	fits_create_img(fptr, bitpix, naxis, naxes, &status);
	fits_write_key(fptr, TSTRING, "CTYPE1", &"RA---SIN", "", &status);
	fits_write_key(fptr, TSTRING, "CTYPE2", &"DEC--SIN", "", &status);
  	keyval=-1.8e2*img[im].imgres/PI;
	fits_write_key(fptr, TDOUBLE, "CDELT1", &keyval, "", &status);
  	keyval=1.8e2*img[im].imgres/PI;
	fits_write_key(fptr, TDOUBLE, "CDELT2", &keyval, "", &status);
  	keyval=(double) img[im].pxls/2+0.5;
	fits_write_key(fptr, TDOUBLE, "CRPIX1", &keyval, "", &status);
	fits_write_key(fptr, TDOUBLE, "CRPIX2", &keyval, "", &status);
  	keyval=0.0;
	fits_write_key(fptr, TDOUBLE, "CRVAL1", &keyval, "", &status);
	fits_write_key(fptr, TDOUBLE, "CRVAL2", &keyval, "", &status);
	fits_write_key(fptr, TSTRING, "CTYPE3", &"VELO-LSR", "", &status);  
  	keyval=img[im].velres;
	fits_write_key(fptr, TDOUBLE, "CDELT3", &keyval, "", &status);
 	keyval=(double) img[im].nchan/2.+1;
	fits_write_key(fptr, TDOUBLE, "CRPIX3", &keyval, "", &status);
  	keyval=0.0;
	fits_write_key(fptr, TDOUBLE, "CRVAL3", &keyval, "", &status);
	if(img[im].unit==0) fits_write_key(fptr, TSTRING, "bunit", &"K", "", &status);
	if(img[im].unit==1) fits_write_key(fptr, TSTRING, "bunit", &"JY/PIXEL", "", &status);
	if(img[im].unit==2) fits_write_key(fptr, TSTRING, "bunit", &"Wm2Hzsr", "", &status);
	if(img[im].unit==3) fits_write_key(fptr, TSTRING, "bunit", &"Lsun/PIXEL", "", &status);
	fits_write_key(fptr, TSTRING, "OBJECT", &"model", "", &status);
	
	for(py=0;py<img[im].pxls;py++){
		for(ichan=0;ichan<img[im].nchan;ichan++){
			for(px=0;px<img[im].pxls;px++){
				if(img[im].unit==0) row[px]=(float) img[im].pixel[px+py*img[im].pxls].intense[ichan]*pow(CLIGHT/img[im].freq,2)/2./KBOLTZ*m[0].norm; 
				if(img[im].unit==1) row[px]=(float) img[im].pixel[px+py*img[im].pxls].intense[ichan]*1e26*img[im].imgres*img[im].imgres*m[0].norm; 
				if(img[im].unit==2) row[px]=(float) img[im].pixel[px+py*img[im].pxls].intense[ichan]*m[0].norm; 
				if(img[im].unit==3) row[px]=(float) img[im].pixel[px+py*img[im].pxls].intense[ichan]*4.*PI*pow(img[im].distance/1.975e13,2)*img[im].freq*img[im].imgres*img[im].imgres*m[0].norm; 
				if(img[im].unit==4) row[px]=(float) img[im].pixel[px+py*img[im].pxls].tau[ichan]; 
#if LINEPOLARIZATION == ON
				if(img[im].unit==13)  row[px]=(float) img[im].pixel[px+py*img[im].pxls].intensepx[ichan]*1e26*img[im].imgres*img[im].imgres*m[0].norm;
				if(img[im].unit==14)  row[px]=(float) img[im].pixel[px+py*img[im].pxls].intensepy[ichan]*1e26*img[im].imgres*img[im].imgres*m[0].norm;
				if(img[im].unit==15)  row[px]=(float) img[im].pixel[px+py*img[im].pxls].intensepz[ichan]*1e26*img[im].imgres*img[im].imgres*m[0].norm;
				if(img[im].unit==16)  row[px]=(float) img[im].pixel[px+py*img[im].pxls].taupx[ichan];
				if(img[im].unit==17)  row[px]=(float) img[im].pixel[px+py*img[im].pxls].taupy[ichan];
				if(img[im].unit==18)  row[px]=(float) img[im].pixel[px+py*img[im].pxls].taupz[ichan];				
#endif
			  if (row[px]<(float) eps) row[px]=(float)eps;
			}
			fpixels[0]=1;
			fpixels[1]=py+1;
			fpixels[2]=ichan+1;
			lpixels[0]=img[im].pxls;
			lpixels[1]=py+1;
			lpixels[2]=ichan+1;
			fits_write_subset(fptr, TFLOAT, fpixels, lpixels, row, &status);
		}
	}
	fits_close_file(fptr, &status);
	free(row);
	if(!silent) done(13);
}
